"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HAPClientConnection = exports.HAPClientConnectionEvents = exports.HAPClient = exports.HAPClientEvents = void 0;
const net_1 = __importDefault(require("net"));
const debug_1 = __importDefault(require("debug"));
const assert_1 = __importDefault(require("assert"));
const crypto_1 = __importDefault(require("crypto"));
const tlv = __importStar(require("./utils/tlv"));
const encryption = __importStar(require("./crypto/encryption"));
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const http_protocol_1 = require("./lib/http-protocol");
const EventEmitter_1 = require("./lib/EventEmitter");
const hap_proxy_1 = require("./types/hap-proxy");
const HAPBonjourBrowser_1 = require("./lib/HAPBonjourBrowser");
const fast_srp_hap_1 = require("fast-srp-hap");
const debug = debug_1.default("HAPClient");
const debugCon = debug_1.default("HAPClient:Connection");
var HAPClientEvents;
(function (HAPClientEvents) {
    HAPClientEvents["CONFIG_NUMBER_CHANGE"] = "config-number-change";
})(HAPClientEvents = exports.HAPClientEvents || (exports.HAPClientEvents = {}));
class HAPClient extends EventEmitter_1.EventEmitter {
    constructor(clientInfo) {
        super();
        this.connections = [];
        this.clientInfo = clientInfo;
        this.bonjourBrowser = new HAPBonjourBrowser_1.HAPBonjourBrowser(clientInfo);
        this.bonjourBrowser.on(HAPBonjourBrowser_1.BonjourBrowserEvents.UPDATE, this.deviceInfoUpdated.bind(this));
        this.bonjourBrowser.on(HAPBonjourBrowser_1.BonjourBrowserEvents.CONFIG_NUMBER_CHANGE, (num) => {
            this.emit(HAPClientEvents.CONFIG_NUMBER_CHANGE, num);
        });
        this.currentChain = Promise.resolve()
            .then(() => this.clientInfo.load()) // transparently load ClientInfo from disk
            .then(() => debug("Loaded ClientInfo for '%s'", this.clientInfo.clientId))
            .then(() => this.bonjourBrowser.deviceInfoPromise()) // TODO startup should fail correctly when this promise returns from timeout
            .then(deviceInfo => this.deviceInfo = deviceInfo);
    }
    deviceInfoUpdated(deviceInfo) {
        this.deviceInfo = deviceInfo;
    }
    newConnection() {
        const connection = new HAPClientConnection(this);
        connection.on(HAPClientConnectionEvents.DISCONNECTED, this.handleDisconnected.bind(this, connection));
        this.connections.push(connection);
        return connection;
    }
    ensureConnected(connection) {
        return this.currentChain = this.currentChain
            .then(() => connection.ensureConnected());
    }
    checkPaired(connection) {
        return this.currentChain = this.currentChain
            .then(() => this.clientInfo.paired ? Promise.resolve() : this.sendPairM1(connection));
    }
    sendPairM1(connection) {
        debugCon("Sending pair setup M1");
        this.pairSetupSession = {
            initiator: connection,
        };
        const startRequest = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M1, hap_proxy_1.TLVValues.METHOD, hap_proxy_1.PairMethods.PAIR_SETUP);
        return connection.sendPairRequest(http_protocol_1.HTTPRoutes.PAIR_SETUP, startRequest)
            .then(this.handlePairM2.bind(this, connection));
    }
    handlePairM2(connection, response) {
        debugCon("Received pair setup M2 response");
        const objects = tlv.decode(response.body);
        const state = objects[hap_proxy_1.TLVValues.STATE][0];
        assert_1.default(state === hap_proxy_1.HAPStates.M2, "Response was not in state M2");
        if (objects[hap_proxy_1.TLVValues.ERROR]) {
            debugCon("M2: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
            connection.disconnect();
            return Promise.reject("M2: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
        }
        const serverPublicKey = objects[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const salt = objects[hap_proxy_1.TLVValues.SALT];
        return this.sendPairM3(connection, serverPublicKey, salt);
    }
    sendPairM3(connection, serverPublicKey, salt) {
        debugCon("Sending pair setup M3");
        return this.clientInfo.pincode().then(pinCode => fast_srp_hap_1.SRP.genKey(32).then(key => {
            const srpParams = fast_srp_hap_1.SRP.params.hap;
            const client = new fast_srp_hap_1.SrpClient(srpParams, salt, Buffer.from("Pair-Setup"), Buffer.from(pinCode), key);
            this.pairSetupSession.srpClient = client;
            client.setB(serverPublicKey);
            const A = client.computeA();
            const M1 = client.computeM1();
            const verifyRequest = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M3, hap_proxy_1.TLVValues.PUBLIC_KEY, A, hap_proxy_1.TLVValues.PASSWORD_PROOF, M1);
            return connection.sendPairRequest(http_protocol_1.HTTPRoutes.PAIR_SETUP, verifyRequest)
                .then(this.handlePairM4.bind(this, connection));
        }));
    }
    handlePairM4(connection, response) {
        debugCon("Received pair setup M4 response");
        const objects = tlv.decode(response.body);
        const state = objects[hap_proxy_1.TLVValues.STATE][0];
        assert_1.default(state === hap_proxy_1.HAPStates.M4, "Response was not in state M4");
        if (objects[hap_proxy_1.TLVValues.ERROR]) {
            debugCon("M4: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
            connection.disconnect();
            return Promise.reject("M4: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
        }
        const session = this.pairSetupSession;
        const srpClient = session.srpClient;
        const serverProof = objects[hap_proxy_1.TLVValues.PASSWORD_PROOF];
        const encryptedData = objects[hap_proxy_1.TLVValues.ENCRYPTED_DATA];
        try {
            srpClient.checkM2(serverProof);
        }
        catch (error) {
            debugCon("ERROR: srp serverProof could not be verified: " + error.message);
            connection.disconnect();
            return Promise.reject("ERROR: srp serverProof could not be verified: " + error.message);
        }
        if (encryptedData) {
            debugCon("Received MFI challenge, ignoring it");
        }
        session.sharedSecret = srpClient.computeK();
        return this.sendPairM5(connection);
    }
    sendPairM5(connection) {
        debugCon("Sending pair setup M5");
        const session = this.pairSetupSession;
        const sharedSecret = session.sharedSecret;
        // step 1 (LT keys already generated)
        // step 2
        let salt = Buffer.from("Pair-Setup-Controller-Sign-Salt");
        let info = Buffer.from("Pair-Setup-Controller-Sign-Info");
        const iOSDeviceX = encryption.HKDF("sha512", salt, sharedSecret, info, 32);
        // step 3
        const iOSDeviceInfo = Buffer.concat([
            iOSDeviceX,
            Buffer.from(this.clientInfo.clientId),
            this.clientInfo.longTermPublicKey,
        ]);
        // step 4
        const iOSDeviceSignature = tweetnacl_1.default.sign.detached(iOSDeviceInfo, this.clientInfo.longTermSecretKey);
        // step 5
        const subTLV = tlv.encode(hap_proxy_1.TLVValues.IDENTIFIER, this.clientInfo.clientId, hap_proxy_1.TLVValues.PUBLIC_KEY, this.clientInfo.longTermPublicKey, hap_proxy_1.TLVValues.SIGNATURE, iOSDeviceSignature);
        // step 6
        salt = Buffer.from("Pair-Setup-Encrypt-Salt");
        info = Buffer.from("Pair-Setup-Encrypt-Info");
        session.sessionKey = encryption.HKDF("sha512", salt, sharedSecret, info, 32);
        const nonce = Buffer.from("PS-Msg05");
        const encrypted = encryption.chacha20_poly1305_encryptAndSeal(session.sessionKey, nonce, null, subTLV);
        const encryptedData = encrypted.ciphertext;
        const authTag = encrypted.authTag;
        // step 7
        const exchangeRequest = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M5, hap_proxy_1.TLVValues.ENCRYPTED_DATA, Buffer.concat([encryptedData, authTag]));
        return connection.sendPairRequest(http_protocol_1.HTTPRoutes.PAIR_SETUP, exchangeRequest)
            .then(this.handlePairM6.bind(this, connection));
    }
    handlePairM6(connection, response) {
        debugCon("Received pair setup M6 response");
        const session = this.pairSetupSession;
        const objects = tlv.decode(response.body);
        const state = objects[hap_proxy_1.TLVValues.STATE][0];
        assert_1.default(state === hap_proxy_1.HAPStates.M6, "Response was not in state M6");
        if (objects[hap_proxy_1.TLVValues.ERROR]) {
            debugCon("M6: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
            connection.disconnect();
            return Promise.reject("M6: received error code " + hap_proxy_1.TLVErrors[objects[hap_proxy_1.TLVValues.ERROR][0]]);
        }
        // step 1 + 2
        const encryptedDataContent = objects[hap_proxy_1.TLVValues.ENCRYPTED_DATA];
        const encryptedData = encryptedDataContent.slice(0, -16);
        const authTag = encryptedDataContent.slice(-16);
        const nonce = Buffer.from("PS-Msg06");
        let plaintextBuffer;
        try {
            plaintextBuffer = encryption.chacha20_poly1305_decryptAndVerify(session.sessionKey, nonce, null, encryptedData, authTag);
        }
        catch (error) {
            debugCon("M6: Could not verify and decrypt: " + error.stack);
            connection.disconnect();
            return Promise.reject("M6: Could not verify and decrypt!");
        }
        const subTLV = tlv.decode(plaintextBuffer);
        const accessoryIdentifier = subTLV[hap_proxy_1.TLVValues.IDENTIFIER];
        const accessoryLTPK = subTLV[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const accessorySignature = subTLV[hap_proxy_1.TLVValues.SIGNATURE];
        // step 3
        const salt = Buffer.from("Pair-Setup-Accessory-Sign-Salt");
        const info = Buffer.from("Pair-Setup-Accessory-Sign-Info");
        const accessoryX = encryption.HKDF("sha512", salt, session.srpClient.computeK(), info, 32);
        const accessoryInfo = Buffer.concat([
            accessoryX,
            accessoryIdentifier,
            accessoryLTPK,
        ]);
        if (!tweetnacl_1.default.sign.detached.verify(accessoryInfo, accessorySignature, accessoryLTPK)) {
            debugCon("M6: Could not verify accessory signature!");
            connection.disconnect();
            return Promise.reject("Could not verify accessory signature!");
        }
        this.clientInfo.accessoryIdentifier = accessoryIdentifier.toString();
        this.clientInfo.accessoryLTPK = accessoryLTPK;
        this.clientInfo.paired = true;
        this.pairSetupSession = undefined;
        debugCon("Successfully paired with %s", accessoryIdentifier.toString());
        return this.clientInfo.save();
    }
    handleDisconnected(connection) {
        // TODO debug("Connection disconnected %s:%d!", connection.remoteAddress.address, connection.remoteAddress.port);
        if (this.pairSetupSession && this.pairSetupSession.initiator === connection) {
            this.pairSetupSession = undefined;
        }
        const index = this.connections.indexOf(connection);
        if (index >= 0) {
            this.connections.splice(index, 1);
        }
    }
}
exports.HAPClient = HAPClient;
var HAPClientConnectionEvents;
(function (HAPClientConnectionEvents) {
    HAPClientConnectionEvents["EVENT_RAW"] = "event-raw";
    HAPClientConnectionEvents["DISCONNECTED"] = "disconnected";
})(HAPClientConnectionEvents = exports.HAPClientConnectionEvents || (exports.HAPClientConnectionEvents = {}));
class HAPClientConnection extends EventEmitter_1.EventEmitter {
    constructor(client) {
        super();
        this.pairingVerified = false;
        this.socketClosed = false;
        this.parser = new http_protocol_1.HTTPResponseParser();
        this.client = client;
        this.clientInfo = client.clientInfo;
        this.connectionChain = Promise.resolve();
    }
    // should not be called directly
    ensureConnected() {
        if (this.socket && !this.socketClosed) {
            return Promise.resolve();
        }
        if (!this.client.deviceInfo) {
            return Promise.reject("ensureConnected() should not be called directly on the connection object!");
        }
        debugCon("Opening socket...");
        this.socketClosed = false;
        this.socket = net_1.default.createConnection(this.client.deviceInfo.port, this.client.deviceInfo.host);
        this.socket.on('data', this.handleIncomingData.bind(this));
        this.socket.on('error', this.handleError.bind(this));
        this.socket.on('close', this.handleClosed.bind(this));
        this.socket.setKeepAlive(true);
        this.socket.setNoDelay(true);
        return new Promise((resolve, reject) => {
            this.socketConnectReject = reject;
            this.socket.on('connect', () => {
                debugCon("Successfully connected!");
                this.socketConnectReject = undefined;
                resolve();
            });
        });
    }
    disconnect() {
        if (this.socket && !this.socketClosed) {
            this.socket.end();
            this.socketClosed = true;
        }
    }
    ensurePairingVerified() {
        return this.connectionChain = this.connectionChain
            .then(() => this.client.ensureConnected(this))
            .then(() => this.client.checkPaired(this))
            .then(() => this.pairingVerified ? Promise.resolve() : this.sendPairVerifyM1());
    }
    accessories() {
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.get(http_protocol_1.HTTPRoutes.ACCESSORIES));
    }
    getCharacteristics(characteristics, event, meta, perms, type) {
        const queryParams = {};
        let id = "";
        characteristics.forEach(request => {
            if (id.length) {
                id += ",";
            }
            id += request.aid + "." + request.iid;
        });
        queryParams["id"] = id;
        if (event) {
            queryParams["ev"] = "1";
        }
        if (meta) {
            queryParams["meta"] = "1";
        }
        if (perms) {
            queryParams["perms"] = "1";
        }
        if (type) {
            queryParams["type"] = "1";
        }
        return this.getCharacteristicsRaw(queryParams);
    }
    getCharacteristicsRaw(queryParams) {
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.get(http_protocol_1.HTTPRoutes.CHARACTERISTICS, queryParams));
    }
    setCharacteristics(characteristics, pid) {
        const request = {
            characteristics: characteristics,
            pid: pid,
        };
        const body = Buffer.from(JSON.stringify(request));
        return this.setCharacteristicsRaw(body);
    }
    setCharacteristicEvents(characteristics, pid) {
        const request = {
            characteristics: characteristics,
            pid: pid,
        };
        const body = Buffer.from(JSON.stringify(request));
        return this.setCharacteristicsRaw(body);
    }
    setCharacteristicsRaw(writeRequest) {
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.put(http_protocol_1.HTTPRoutes.CHARACTERISTICS, writeRequest));
    }
    prepareWrite(ttl, pid) {
        if (pid === undefined) {
            pid = this.generatePid();
        }
        const body = Buffer.from(JSON.stringify({
            ttl: ttl,
            pid: pid,
        }));
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.put(http_protocol_1.HTTPRoutes.PREPARE, body))
            .then(response => new Promise((resolve, reject) => {
            const responseStatus = JSON.parse(response.body.toString());
            if (responseStatus.status === 0) {
                resolve(pid);
            }
            else {
                reject(responseStatus.status);
            }
        }));
    }
    prepareWriteRaw(prepareRequest) {
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.put(http_protocol_1.HTTPRoutes.PREPARE, prepareRequest));
    }
    generatePid() {
        const ran = crypto_1.default.randomBytes(2);
        return ran.readUInt16LE(0);
    }
    // TODO /resource
    resourceRaw(resourceRequest) {
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.post(http_protocol_1.HTTPRoutes.RESOURCE, resourceRequest));
    }
    // TODO addPairing
    // TODO listPairings
    /**
     * Sends the request to remove a pairing. When clientId is omitted the own pairing will be removed.
     *
     * @param clientInfo {ClientInfo} - clientInfo of the pairing which will be removed (optional, defaults to own id)
     */
    removePairing(clientInfo = this.clientInfo) {
        // step 1
        if (!clientInfo) {
            clientInfo = this.clientInfo;
        }
        // step 2
        const requestTLV = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M1, hap_proxy_1.TLVValues.METHOD, hap_proxy_1.PairMethods.REMOVE_PAIRING, hap_proxy_1.TLVValues.IDENTIFIER, Buffer.from(clientInfo.clientId));
        // step 3
        return this.connectionChain = this.ensurePairingVerified()
            .then(() => this.sendPairRequest(http_protocol_1.HTTPRoutes.PAIRINGS, requestTLV))
            .then((response) => {
            const body = response.body;
            const responseTlv = tlv.decode(body);
            const error = responseTlv[hap_proxy_1.TLVValues.ERROR];
            if (error) {
                const errorCode = error.readUInt8(0);
                debug("Error removing pairing: " + hap_proxy_1.TLVErrors[errorCode]);
                throw new Error(errorCode + "");
            }
            else {
                clientInfo.paired = false;
                return clientInfo.save();
            }
        });
    }
    sendPairVerifyM1() {
        debugCon("Sending pair-verify M1");
        const keyPair = encryption.generateCurve25519KeyPair();
        const secretKey = Buffer.from(keyPair.secretKey);
        const publicKey = Buffer.from(keyPair.publicKey);
        this.pairVerifySession = {
            secretKey: secretKey,
            publicKey: publicKey,
        };
        const startRequest = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M1, hap_proxy_1.TLVValues.PUBLIC_KEY, publicKey);
        return this.sendPairRequest(http_protocol_1.HTTPRoutes.PAIR_VERIFY, startRequest)
            .then(this.handlePairVerifyM2.bind(this));
    }
    handlePairVerifyM2(response) {
        debugCon("Received pair-verify M2");
        const objects = tlv.decode(response.body);
        const state = objects[hap_proxy_1.TLVValues.STATE].readUInt8(0);
        const error = objects[hap_proxy_1.TLVValues.ERROR];
        const serverPublicKey = objects[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const encryptedData = objects[hap_proxy_1.TLVValues.ENCRYPTED_DATA];
        assert_1.default.strictEqual(state, hap_proxy_1.HAPStates.M2, "PairVerify state did not match M2");
        if (error) {
            debugCon("Pair-Verify M2 returned with error: " + hap_proxy_1.TLVErrors[error[0]]);
            this.disconnect();
            return Promise.reject("Pair-Verify M2 returned with error: " + hap_proxy_1.TLVErrors[error[0]]);
        }
        assert_1.default.strictEqual(serverPublicKey.length, 32, "serverPublicKey must be 32 bytes");
        const session = this.pairVerifySession;
        // Step 1
        const sharedSecret = Buffer.from(encryption.generateCurve25519SharedSecKey(session.secretKey, serverPublicKey));
        // Step 2
        const encryptionSalt = Buffer.from("Pair-Verify-Encrypt-Salt");
        const encryptionInfo = Buffer.from("Pair-Verify-Encrypt-Info");
        const sessionKey = encryption.HKDF("sha512", encryptionSalt, sharedSecret, encryptionInfo, 32);
        session.sharedSecret = sharedSecret;
        // Step 3 & 4
        const cipherText = encryptedData.slice(0, -16);
        const authTag = encryptedData.slice(-16);
        const nonce = Buffer.from("PV-Msg02");
        let plaintext;
        try {
            plaintext = encryption.chacha20_poly1305_decryptAndVerify(sessionKey, nonce, null, cipherText, authTag);
        }
        catch (error) {
            console.error("WARNING: M2 - Could not verify cipherText: " + error.stack);
            this.disconnect();
            return Promise.reject("WARNING: M2 - Could not verify cipherText");
        }
        // Step 5
        const data = tlv.decode(plaintext);
        const accessoryIdentifier = data[hap_proxy_1.TLVValues.IDENTIFIER];
        const accessorySignature = data[hap_proxy_1.TLVValues.SIGNATURE];
        // we would need to lookup our pairing (retrieve accessoryLTPK).
        // We do however only support one pairing, thus check if ids match
        if (this.clientInfo.accessoryIdentifier !== accessoryIdentifier.toString()) {
            console.error("WARNING: identifier is not the expected store in the keystore");
            this.disconnect();
            return Promise.reject("WARNING: identifier is not the expected store in the keystore");
        }
        // Step 6
        const accessoryInfo = Buffer.concat([
            serverPublicKey,
            accessoryIdentifier,
            session.publicKey
        ]);
        if (!tweetnacl_1.default.sign.detached.verify(accessoryInfo, accessorySignature, this.clientInfo.accessoryLTPK)) {
            debugCon("M2: Failed in pair-verify to verify accessory signature!");
            this.disconnect();
            return Promise.reject("M2: Failed in pair-verify to verify accessory signature!");
        }
        return this.sendPairVerifyM3(serverPublicKey, sessionKey);
    }
    sendPairVerifyM3(serverPublicKey, encryptionKey) {
        debugCon("Sending pair-verify M3");
        const session = this.pairVerifySession;
        // Step 7
        const iOSDeviceInfo = Buffer.concat([
            session.publicKey,
            Buffer.from(this.clientInfo.clientId),
            serverPublicKey,
        ]);
        // Step 8
        const iOSDeviceSignature = Buffer.from(tweetnacl_1.default.sign.detached(iOSDeviceInfo, this.clientInfo.longTermSecretKey));
        // Step 9
        const plainTextTLV = tlv.encode(hap_proxy_1.TLVValues.IDENTIFIER, this.clientInfo.clientId, hap_proxy_1.TLVValues.SIGNATURE, iOSDeviceSignature);
        // Step 10
        const nonce = Buffer.from("PV-Msg03");
        const encrypted = encryption.chacha20_poly1305_encryptAndSeal(encryptionKey, nonce, null, plainTextTLV);
        // Step 11
        const finishRequest = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M3, hap_proxy_1.TLVValues.ENCRYPTED_DATA, Buffer.concat([encrypted.ciphertext, encrypted.authTag]));
        // Step 12
        return this.sendPairRequest(http_protocol_1.HTTPRoutes.PAIR_VERIFY, finishRequest)
            .then(this.handlePairVerifyM4.bind(this));
    }
    handlePairVerifyM4(response) {
        debugCon("Received pair-verify step M4");
        const objects = tlv.decode(response.body);
        const state = objects[hap_proxy_1.TLVValues.STATE].readUInt8(0);
        const error = objects[hap_proxy_1.TLVValues.ERROR];
        assert_1.default.strictEqual(state, hap_proxy_1.HAPStates.M4, "PairVerify state did not match M4");
        const session = this.pairVerifySession;
        const sharedSecret = session.sharedSecret;
        this.pairVerifySession = undefined;
        if (error) {
            debugCon("Pair-Verify was unsuccessful: " + hap_proxy_1.TLVErrors[error[0]]);
            this.disconnect();
            return Promise.reject("pair-verify was unsuccessful: " + hap_proxy_1.TLVErrors[error[0]]);
        }
        else {
            // generate HAP encryption/decryption keys
            const salt = Buffer.from("Control-Salt");
            const accessoryToControllerInfo = Buffer.from("Control-Read-Encryption-Key");
            const controllerToAccessoryInfo = Buffer.from("Control-Write-Encryption-Key");
            const accessoryToControllerKey = encryption.HKDF("sha512", salt, sharedSecret, accessoryToControllerInfo, 32);
            const controllerToAccessoryKey = encryption.HKDF("sha512", salt, sharedSecret, controllerToAccessoryInfo, 32);
            this.encryptionContext = new hap_proxy_1.HAPEncryptionContext(sharedSecret, controllerToAccessoryKey, accessoryToControllerKey);
            this.pairingVerified = true;
            debugCon("Pair-Verify was successful");
            return Promise.resolve();
        }
    }
    handleIncomingData(data) {
        if (this.socketClosed) {
            return;
        }
        if (this.encryptionContext) {
            try {
                data = encryption.layerDecrypt(data, this.encryptionContext);
            }
            catch (error) {
                // decryption failed
                this.disconnect();
                return;
            }
        }
        this.parser.appendData(data);
        const messages = this.parser.parse();
        messages.forEach(message => {
            if (message.messageType === "EVENT") {
                this.emit(HAPClientConnectionEvents.EVENT_RAW, message.body);
            }
            else if (message.messageType === "HTTP") {
                if (this.httpRequestResolver) {
                    const resolve = this.httpRequestResolver;
                    this.httpRequestResolver = undefined;
                    resolve(message);
                }
                else {
                    console.error("WARNING: Received http response when not expecting anything!");
                }
            }
            else {
                console.error("WARNING: Received unknown message type!");
            }
        });
    }
    get(route, queryParams = {}, headers = {}) {
        return this.sendRequest(http_protocol_1.HTTPMethod.GET, route, http_protocol_1.HTTPContentType.HAP_JSON, queryParams, headers);
    }
    put(route, data, queryParams = {}, headers = {}) {
        return this.sendRequest(http_protocol_1.HTTPMethod.PUT, route, http_protocol_1.HTTPContentType.HAP_JSON, queryParams, headers, data);
    }
    post(route, data, queryParams = {}, headers = {}) {
        return this.sendRequest(http_protocol_1.HTTPMethod.POST, route, http_protocol_1.HTTPContentType.HAP_JSON, queryParams, headers, data);
    }
    sendPairRequest(route, data) {
        return this.sendRequest(http_protocol_1.HTTPMethod.POST, route, http_protocol_1.HTTPContentType.PAIRING_TLV8, {}, {}, data);
    }
    sendRequest(method, route, contentType, queryParams = {}, headers = {}, data) {
        if (!this.socket || this.socketClosed) {
            return Promise.reject("Connection was not established!");
        }
        if (this.httpRequestResolver) {
            console.error("WARNING: http request still in progress");
            return Promise.reject("Request still in progress");
        }
        headers["Content-Type"] = contentType;
        if (data && (method === http_protocol_1.HTTPMethod.POST || method === http_protocol_1.HTTPMethod.PUT)) {
            headers["Content-Length"] = data.length + "";
        }
        else {
            data = Buffer.alloc(0);
        }
        let query = "";
        Object.entries(queryParams).forEach(([key, value]) => {
            if (typeof value === "string") {
                if (query.length) {
                    query += ",";
                }
                query += key + "=" + value;
            }
            else {
                value.forEach(value0 => {
                    if (query.length) {
                        query += ",";
                    }
                    query += key + "=" + value0;
                });
            }
        });
        if (query) {
            query = "?" + query;
        }
        let request = Buffer.concat([
            Buffer.from(`${method} ${route + query} HTTP/1.1\r\n` +
                `Host: ${this.client.deviceInfo.host}:${this.client.deviceInfo.port}\r\n` +
                Object.keys(headers).reduce((acc, header) => {
                    return acc + `${header}: ${headers[header]}\r\n`;
                }, "") +
                `\r\n` // additional newline before content
            ),
            data
        ]);
        if (this.encryptionContext) {
            request = encryption.layerEncrypt(request, this.encryptionContext);
        }
        return new Promise(resolve => {
            this.httpRequestResolver = resolve;
            this.socket.write(request);
        });
    }
    handleError(error) {
        // TODO log
        if (this.socketConnectReject) {
            this.socketConnectReject(error);
        }
    }
    handleClosed() {
        this.socket = undefined;
        this.pairVerifySession = undefined;
        this.pairingVerified = false;
        this.encryptionContext = undefined;
        this.httpRequestResolver = undefined;
        this.parser = new http_protocol_1.HTTPResponseParser(); // ensure parser buffer is cleared up
        debugCon("Disconnected!"); // TODO debug
        this.emit(HAPClientConnectionEvents.DISCONNECTED);
    }
}
exports.HAPClientConnection = HAPClientConnection;
//# sourceMappingURL=HAPClient.js.map