"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HAPServerConnection = exports.NOT_FOUND_HANDLER = exports.HAPServerConnectionEvents = exports.HAPServer = exports.HAPServerEvents = void 0;
const hap_proxy_1 = require("./types/hap-proxy");
const net_1 = __importDefault(require("net"));
const debug_1 = __importDefault(require("debug"));
const encryption = __importStar(require("./crypto/encryption"));
const http_protocol_1 = require("./lib/http-protocol");
const EventEmitter_1 = require("./lib/EventEmitter");
const crypto = __importStar(require("crypto"));
const tlv = __importStar(require("./utils/tlv"));
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const Advertiser_1 = require("./lib/Advertiser");
const AccessoryInfo_1 = require("./storage/AccessoryInfo");
const once_1 = require("./utils/once");
const url = __importStar(require("url"));
const uuid_1 = require("./utils/uuid");
const fast_srp_hap_1 = require("fast-srp-hap");
const debug = debug_1.default("HAPServer");
const debugCon = debug_1.default("HAPProxy");
var HAPServerEvents;
(function (HAPServerEvents) {
    HAPServerEvents["CONNECTION"] = "connection";
    HAPServerEvents["ACCESSORIES"] = "accessories";
    HAPServerEvents["GET_CHARACTERISTICS"] = "get-characteristics";
    HAPServerEvents["SET_CHARACTERISTICS"] = "set-characteristics";
    HAPServerEvents["PREPARE_WRITE"] = "prepare";
    HAPServerEvents["RESOURCE"] = "resource";
    HAPServerEvents["PAIRED"] = "paired";
    HAPServerEvents["UNPAIRED"] = "unpaired";
})(HAPServerEvents = exports.HAPServerEvents || (exports.HAPServerEvents = {}));
class HAPServer extends EventEmitter_1.EventEmitter {
    constructor(accessoryInfo) {
        super();
        this.connections = [];
        this.routeHandlers = {
            [http_protocol_1.HTTPRoutes.IDENTIFY]: this.handleIdentify.bind(this),
            [http_protocol_1.HTTPRoutes.PAIR_SETUP]: this.handlePair.bind(this),
            [http_protocol_1.HTTPRoutes.PAIR_VERIFY]: this.handlePairVerify.bind(this),
            [http_protocol_1.HTTPRoutes.PAIRINGS]: this.handlePairings.bind(this),
            [http_protocol_1.HTTPRoutes.ACCESSORIES]: this.handleAccessories.bind(this),
            [http_protocol_1.HTTPRoutes.CHARACTERISTICS]: this.handleCharacteristics.bind(this),
            [http_protocol_1.HTTPRoutes.PREPARE]: this.handlePrepare.bind(this),
            [http_protocol_1.HTTPRoutes.RESOURCE]: this.handleResource.bind(this),
        };
        this.accessoryInfo = accessoryInfo;
        this.accessoryInfo.on(AccessoryInfo_1.AccessoryInfoEvents.REMOVED_CLIENT, this.handleClientUnpaired.bind(this));
        this.tcpServer = net_1.default.createServer();
        this.tcpServer.on('listening', this.handleListening.bind(this));
        this.tcpServer.on('connection', this.handleConnection.bind(this));
        this.advertiser = new Advertiser_1.Advertiser(accessoryInfo);
    }
    listen(targetPort) {
        this.accessoryInfo.load() // transparently load AccessoryInfo from disk
            .then(() => debug("Loaded AccessoryInfo for '%s'", this.accessoryInfo.displayName))
            .then(() => this.tcpServer.listen(targetPort));
    }
    stop() {
        this.tcpServer.close();
        this.connections.forEach(connection => connection.disconnect());
    }
    handleListening() {
        const address = this.tcpServer.address();
        if (address && typeof address !== "string") {
            const port = address.port;
            debug("Server listening on port %s", port);
            this.advertiser.createService(port);
            this.advertiser.startAdvertising();
        }
    }
    handleConnection(socket) {
        const connection = new HAPServerConnection(socket, this.routeHandlers);
        connection.on(HAPServerConnectionEvents.DISCONNECTED, this.handleConnectionClosed.bind(this, connection));
        this.connections.push(connection);
        debug("Received new connection on %s!", connection.remoteAddress);
        this.emit(HAPServerEvents.CONNECTION, connection);
    }
    handleConnectionClosed(connection) {
        debug("Connection disconnected %s!", connection.remoteAddress);
        const index = this.connections.indexOf(connection);
        if (index >= 0) {
            this.connections.splice(index, 1);
        }
        if (this.currentPairSetupSession && this.currentPairSetupSession.connection === connection) {
            this.currentPairSetupSession = undefined;
        }
    }
    handleClientUnpaired(initiator, clientId) {
        this.connections.forEach(connection => {
            if (connection.clientId !== clientId) {
                return;
            }
            if (connection === initiator) {
                // the session which initiated the unpair removed it's own username, wait until the unpair request is finished
                // until we kill his connection
                connection.disconnectAfterWrite();
            }
            else {
                connection.disconnect();
            }
        });
    }
    // noinspection JSMethodCanBeStatic
    handleIdentify(connection, request) {
        if (this.accessoryInfo.hasPairings()) { // if we are paired, a client cannot make an unpaired identify
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        debug("**Unpaired Identify received**");
        // TODO we could potentially forward this but is this really necessary?
        return Promise.resolve({
            status: 204,
            contentType: http_protocol_1.HTTPContentType.HAP_JSON,
        });
    }
    handlePair(connection, request) {
        if (this.accessoryInfo.hasPairings()) {
            return Promise.resolve({
                status: http_protocol_1.HTTPStatus.SUCCESS,
                contentType: http_protocol_1.HTTPContentType.PAIRING_TLV8,
                data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, hap_proxy_1.TLVValues.ERROR, hap_proxy_1.TLVErrors.UNAVAILABLE),
            });
        }
        const data = tlv.decode(request.body);
        const state = data[hap_proxy_1.TLVValues.STATE].readUInt8(0);
        if (this.currentPairSetupSession && this.currentPairSetupSession.connection !== connection) {
            return Promise.resolve({
                status: http_protocol_1.HTTPStatus.SUCCESS,
                contentType: http_protocol_1.HTTPContentType.PAIRING_TLV8,
                data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, hap_proxy_1.TLVValues.ERROR, hap_proxy_1.TLVErrors.UNAVAILABLE),
            });
        }
        if (!this.currentPairSetupSession) {
            this.currentPairSetupSession = {
                connection: connection,
                nextState: hap_proxy_1.HAPStates.M1,
            };
        }
        if (this.currentPairSetupSession && this.currentPairSetupSession.nextState !== state) {
            debug("Received /pair-setup with unexpected state!"); // TODO debug message
            return this.abortPairing(state + 1, hap_proxy_1.TLVErrors.INVALID_REQUEST, http_protocol_1.HTTPStatus.BAD_REQUEST);
        }
        let partialPromise;
        if (state === hap_proxy_1.HAPStates.M1) {
            partialPromise = this.handlePairM1(data);
        }
        else if (state === hap_proxy_1.HAPStates.M3) {
            partialPromise = this.handlePairM3(data);
        }
        else if (state === hap_proxy_1.HAPStates.M5) {
            partialPromise = this.handlePairM5(data);
        }
        else {
            return this.abortPairing(state + 1, hap_proxy_1.TLVErrors.INVALID_REQUEST);
        }
        return partialPromise.then(partial => ({
            status: partial.status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: partial.contentType || http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: partial.data,
            headers: partial.headers,
        }));
    }
    abortPairing(state, error, status) {
        this.currentPairSetupSession = undefined;
        return Promise.resolve({
            status: status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, state, hap_proxy_1.TLVValues.ERROR, error),
        });
    }
    handlePairM1(tlvData) {
        debug("Received pair setup M1");
        const session = this.currentPairSetupSession;
        const method = tlvData[hap_proxy_1.TLVValues.METHOD].readUInt8(0); // TODO handle method
        // TODO pairings flags
        // step 1 - handled above
        // step 2 - not really useful for our scenario (render device unpairable after 100 failed pair attempts)
        // step 3 - handled above
        // step 4
        return fast_srp_hap_1.SRP.genKey(32).then(key => {
            if (session !== this.currentPairSetupSession) {
                throw new Error("pair session changed!");
            }
            // step 5
            const username = Buffer.from("Pair-Setup");
            // step 6
            const salt = crypto.randomBytes(16);
            const srpParams = fast_srp_hap_1.SRP.params.hap;
            // TODO handle pairing flags
            session.srpServer = new fast_srp_hap_1.SrpServer(srpParams, salt, username, Buffer.from(this.accessoryInfo.pincode), key);
            // step 9
            const publicKey = session.srpServer.computeB();
            // step 10
            const response = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, hap_proxy_1.TLVValues.PUBLIC_KEY, publicKey, hap_proxy_1.TLVValues.SALT, salt);
            session.nextState = hap_proxy_1.HAPStates.M3; // we expect M3 for next request
            return {
                data: response,
            };
        });
    }
    handlePairM3(tlvData) {
        debug("Received pair setup M3");
        const session = this.currentPairSetupSession;
        const srpServer = session.srpServer;
        const publicKey = tlvData[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const clientProof = tlvData[hap_proxy_1.TLVValues.PASSWORD_PROOF];
        // step 1
        srpServer.setA(publicKey);
        session.sharedSecret = srpServer.computeK();
        // step 2
        try {
            srpServer.checkM1(clientProof);
        }
        catch (error) {
            console.warn("Rejected server pairing as incorrect pin code was supplied!");
            return this.abortPairing(hap_proxy_1.HAPStates.M4, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 3
        const serverProof = srpServer.computeM2();
        session.nextState = hap_proxy_1.HAPStates.M5;
        return Promise.resolve({
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M4, hap_proxy_1.TLVValues.PASSWORD_PROOF, serverProof)
        });
    }
    handlePairM5(tlvData) {
        debug("Received pair setup M5");
        const session = this.currentPairSetupSession;
        // M5 Verification
        // step 1
        const encryptedDataContent = tlvData[hap_proxy_1.TLVValues.ENCRYPTED_DATA];
        const encryptedData = encryptedDataContent.slice(0, -16);
        const authTag = encryptedDataContent.slice(-16);
        // step 2
        let salt = Buffer.from("Pair-Setup-Encrypt-Salt");
        let info = Buffer.from("Pair-Setup-Encrypt-Info");
        session.sessionKey = encryption.HKDF("sha512", salt, session.sharedSecret, info, 32);
        const nonce = Buffer.from("PS-Msg05");
        let plaintext;
        try {
            plaintext = encryption.chacha20_poly1305_decryptAndVerify(session.sessionKey, nonce, null, encryptedData, authTag);
        }
        catch (error) {
            // TODO debug
            return this.abortPairing(hap_proxy_1.HAPStates.M6, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 3
        salt = Buffer.from("Pair-Setup-Controller-Sign-Salt");
        info = Buffer.from("Pair-Setup-Controller-Sign-Info");
        const iOSDeviceX = encryption.HKDF("sha512", salt, session.sharedSecret, info, 32);
        // step 4
        const subTLV = tlv.decode(plaintext);
        const iOSDevicePairingID = subTLV[hap_proxy_1.TLVValues.IDENTIFIER];
        const iOSDeviceLTPK = subTLV[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const iOSDeviceSignature = subTLV[hap_proxy_1.TLVValues.SIGNATURE];
        const iOSDeviceInfo = Buffer.concat([
            iOSDeviceX,
            iOSDevicePairingID,
            iOSDeviceLTPK,
        ]);
        // step 5
        if (!tweetnacl_1.default.sign.detached.verify(iOSDeviceInfo, iOSDeviceSignature, iOSDeviceLTPK)) {
            // TODO debug
            return this.abortPairing(hap_proxy_1.HAPStates.M6, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 6
        session.clientId = iOSDevicePairingID.toString();
        session.clientLTPK = iOSDeviceLTPK;
        return this.generatePairM5Response();
    }
    generatePairM5Response() {
        debug("Sending pair setup M6");
        const session = this.currentPairSetupSession;
        // step 1 (LT keys already generated)
        // step 2
        const salt = Buffer.from("Pair-Setup-Accessory-Sign-Salt");
        const info = Buffer.from("Pair-Setup-Accessory-Sign-Info");
        const accessoryX = encryption.HKDF("sha512", salt, session.sharedSecret, info, 32);
        // step 3
        const accessoryInfo = Buffer.concat([
            accessoryX,
            Buffer.from(this.accessoryInfo.accessoryId),
            this.accessoryInfo.longTermPublicKey,
        ]);
        // step 4
        const accessorySignature = tweetnacl_1.default.sign.detached(accessoryInfo, this.accessoryInfo.longTermSecretKey);
        // step 5
        const subTLV = tlv.encode(hap_proxy_1.TLVValues.IDENTIFIER, this.accessoryInfo.accessoryId, hap_proxy_1.TLVValues.PUBLIC_KEY, this.accessoryInfo.longTermPublicKey, hap_proxy_1.TLVValues.SIGNATURE, accessorySignature);
        // step 6
        const nonce = Buffer.from("PS-Msg06");
        const encrypted = encryption.chacha20_poly1305_encryptAndSeal(session.sessionKey, nonce, null, subTLV);
        // step 7
        const exchangeResponse = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M6, hap_proxy_1.TLVValues.ENCRYPTED_DATA, Buffer.concat([encrypted.ciphertext, encrypted.authTag]));
        // initial pairing is always the admin pairing
        this.accessoryInfo.addPairedClient(session.clientId, session.clientLTPK, AccessoryInfo_1.PermissionTypes.ADMIN);
        this.currentPairSetupSession = undefined;
        this.advertiser.updateAdvertisement();
        debug("Successfully paired!"); // TODO debug message
        this.emit(HAPServerEvents.PAIRED);
        return this.accessoryInfo.save()
            .then(() => Promise.resolve({
            data: exchangeResponse,
        }));
    }
    handlePairVerify(connection, request) {
        const data = tlv.decode(request.body);
        const state = data[hap_proxy_1.TLVValues.STATE].readUInt8(0);
        if (!connection.pairVerifySession) {
            connection.pairVerifySession = {
                nextState: hap_proxy_1.HAPStates.M1,
            };
        }
        if (connection.pairVerifySession.nextState !== state) {
            // probably received subsequent pair-verify, teardown old session
            connection.pairVerifySession = {
                nextState: hap_proxy_1.HAPStates.M1,
            };
        }
        let partialPromise;
        if (state === hap_proxy_1.HAPStates.M1) {
            partialPromise = this.handlePairVerifyM1(connection, data);
        }
        else if (state === hap_proxy_1.HAPStates.M3) {
            partialPromise = this.handlePairVerifyM3(connection, data);
        }
        else {
            return this.abortPairVerify(connection, state + 1, hap_proxy_1.TLVErrors.INVALID_REQUEST);
        }
        return partialPromise.then(partial => ({
            status: partial.status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: partial.contentType || http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: partial.data,
            headers: partial.headers,
        }));
    }
    // noinspection JSMethodCanBeStatic
    abortPairVerify(connection, state, error, status) {
        connection.pairVerifySession = undefined;
        return Promise.resolve({
            status: status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, state, hap_proxy_1.TLVValues.ERROR, error),
        });
    }
    handlePairVerifyM1(connection, tlvData) {
        debug("Received pair verify M1"); // TODO specify
        const session = connection.pairVerifySession;
        session.clientPublicKey = tlvData[hap_proxy_1.TLVValues.PUBLIC_KEY];
        // step 1
        const keyPair = encryption.generateCurve25519KeyPair();
        session.publicKey = Buffer.from(keyPair.publicKey);
        session.secretKey = Buffer.from(keyPair.secretKey);
        // step 2
        session.sharedSecret = Buffer.from(encryption.generateCurve25519SharedSecKey(session.secretKey, session.clientPublicKey));
        // step 3
        const accessoryInfo = Buffer.concat([
            session.publicKey,
            Buffer.from(this.accessoryInfo.accessoryId),
            session.clientPublicKey,
        ]);
        // step 4
        const accessorySignature = tweetnacl_1.default.sign.detached(accessoryInfo, this.accessoryInfo.longTermSecretKey);
        // step 5
        const subTLV = tlv.encode(hap_proxy_1.TLVValues.IDENTIFIER, this.accessoryInfo.accessoryId, hap_proxy_1.TLVValues.SIGNATURE, accessorySignature);
        // step 6
        const salt = Buffer.from("Pair-Verify-Encrypt-Salt");
        const info = Buffer.from("Pair-Verify-Encrypt-Info");
        session.sessionKey = encryption.HKDF("sha512", salt, session.sharedSecret, info, 32);
        // step 7
        const nonce = Buffer.from("PV-Msg02");
        const encrypted = encryption.chacha20_poly1305_encryptAndSeal(session.sessionKey, nonce, null, subTLV);
        // step 8
        const startResponse = tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, hap_proxy_1.TLVValues.PUBLIC_KEY, session.publicKey, hap_proxy_1.TLVValues.ENCRYPTED_DATA, Buffer.concat([encrypted.ciphertext, encrypted.authTag]));
        session.nextState = hap_proxy_1.HAPStates.M3;
        return Promise.resolve({
            data: startResponse,
        });
    }
    handlePairVerifyM3(connection, tlvData) {
        debug("Received pair verify M3"); // TODO specify
        const session = connection.pairVerifySession;
        // step 1 & 2
        const encryptedDataContent = tlvData[hap_proxy_1.TLVValues.ENCRYPTED_DATA];
        const encryptedData = encryptedDataContent.slice(0, -16);
        const authTag = encryptedDataContent.slice(-16);
        const nonce = Buffer.from("PV-Msg03");
        let plaintext;
        try {
            plaintext = encryption.chacha20_poly1305_decryptAndVerify(session.sessionKey, nonce, null, encryptedData, authTag);
        }
        catch (error) {
            return this.abortPairVerify(connection, hap_proxy_1.HAPStates.M4, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 3
        const subTLV = tlv.decode(plaintext);
        const iOSDevicePairingID = subTLV[hap_proxy_1.TLVValues.IDENTIFIER]; // aka clientId
        const iOSDeviceSignature = subTLV[hap_proxy_1.TLVValues.SIGNATURE];
        const iOSDeviceLTPK = this.accessoryInfo.getClientPublicKey(iOSDevicePairingID.toString());
        if (!iOSDeviceLTPK) {
            // TODO debug client not paired attempted to pair-verify
            return this.abortPairVerify(connection, hap_proxy_1.HAPStates.M4, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 4
        const iOSDeviceInfo = Buffer.concat([
            session.clientPublicKey,
            iOSDevicePairingID,
            session.publicKey,
        ]);
        if (!tweetnacl_1.default.sign.detached.verify(iOSDeviceInfo, iOSDeviceSignature, iOSDeviceLTPK)) {
            // TODO debug: client provided invalid signature
            return this.abortPairVerify(connection, hap_proxy_1.HAPStates.M4, hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 5
        // generate HAP encryption/decryption keys
        const salt = Buffer.from("Control-Salt");
        const accessoryToControllerInfo = Buffer.from("Control-Read-Encryption-Key");
        const controllerToAccessoryInfo = Buffer.from("Control-Write-Encryption-Key");
        const accessoryToControllerKey = encryption.HKDF("sha512", salt, session.sharedSecret, accessoryToControllerInfo, 32);
        const controllerToAccessoryKey = encryption.HKDF("sha512", salt, session.sharedSecret, controllerToAccessoryInfo, 32);
        connection.encryptionContext = new hap_proxy_1.HAPEncryptionContext(session.sharedSecret, accessoryToControllerKey, controllerToAccessoryKey);
        connection.pairingVerified = true;
        connection.clientId = iOSDevicePairingID.toString();
        connection.pairVerifySession = undefined;
        debug("Pairing verified"); // TODO debug message
        return Promise.resolve({
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M4),
        });
    }
    handlePairings(connection, request) {
        if (!connection.pairingVerified) {
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        const tlvData = tlv.decode(request.body);
        const state = tlvData[hap_proxy_1.TLVValues.STATE].readUInt8(0);
        const method = tlvData[hap_proxy_1.TLVValues.METHOD].readUInt8(0);
        if (state !== hap_proxy_1.HAPStates.M1) {
            return this.abortPairings(hap_proxy_1.TLVErrors.INVALID_REQUEST);
        }
        let partialPromise;
        if (method === hap_proxy_1.PairMethods.ADD_PAIRING) {
            partialPromise = this.addPairing(connection, tlvData);
        }
        else if (method === hap_proxy_1.PairMethods.REMOVE_PAIRING) {
            partialPromise = this.removePairing(connection, tlvData);
        }
        else if (method === hap_proxy_1.PairMethods.LIST_PAIRINGS) {
            partialPromise = this.listPairings(connection);
        }
        else {
            return this.abortPairings(hap_proxy_1.TLVErrors.INVALID_REQUEST);
        }
        return partialPromise.then(partial => ({
            status: partial.status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: partial.contentType || http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: partial.data,
            headers: partial.headers,
        }));
    }
    // noinspection JSMethodCanBeStatic
    abortPairings(error, status) {
        return Promise.resolve({
            status: status || http_protocol_1.HTTPStatus.SUCCESS,
            contentType: http_protocol_1.HTTPContentType.PAIRING_TLV8,
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, hap_proxy_1.TLVValues.ERROR, error),
        });
    }
    addPairing(connection, tlvData) {
        const clientId = tlvData[hap_proxy_1.TLVValues.IDENTIFIER].toString();
        const publicKey = tlvData[hap_proxy_1.TLVValues.PUBLIC_KEY];
        const permission = tlvData[hap_proxy_1.TLVValues.PERMISSIONS].readUInt8(0);
        // step 2
        if (!this.accessoryInfo.hasAdminPermissions(connection.clientId)) {
            return this.abortPairings(hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 3
        const clientPublicKey = this.accessoryInfo.getClientPublicKey(clientId);
        if (clientPublicKey) {
            // (a)
            if (clientPublicKey.toString("hex") !== publicKey.toString("hex")) {
                return this.abortPairings(hap_proxy_1.TLVErrors.UNKNOWN);
            }
            // (b)
            this.accessoryInfo.updatePermission(clientId, permission);
        }
        else {
            // step 4
            this.accessoryInfo.addPairedClient(clientId, publicKey, permission);
        }
        // step 5
        // TODO log, additional pairing logged
        return this.accessoryInfo.save()
            .then(() => Promise.resolve({
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2),
        }));
    }
    removePairing(connection, tlvData) {
        const clientId = tlvData[hap_proxy_1.TLVValues.IDENTIFIER].toString();
        // step 2
        if (!this.accessoryInfo.hasAdminPermissions(connection.clientId)) {
            return this.abortPairings(hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        // step 3
        this.accessoryInfo.removePairedClient(connection, clientId);
        if (!this.accessoryInfo.hasPairings()) {
            this.advertiser.updateAdvertisement();
            this.emit(HAPServerEvents.UNPAIRED);
        }
        // TODO log pairing removed
        return this.accessoryInfo.save()
            .then(() => Promise.resolve({
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2),
        }));
    }
    listPairings(connection) {
        if (!this.accessoryInfo.hasAdminPermissions(connection.clientId)) {
            return this.abortPairings(hap_proxy_1.TLVErrors.AUTHENTICATION);
        }
        const pairings = this.accessoryInfo.listPairings();
        const tlvList = [];
        pairings.forEach(pairing => {
            if (tlvList.length > 0) {
                tlvList.push(hap_proxy_1.TLVValues.SEPARATOR, Buffer.alloc(0));
            }
            tlvList.push(hap_proxy_1.TLVValues.IDENTIFIER, pairing.clientId, hap_proxy_1.TLVValues.PUBLIC_KEY, pairing.publicKey, hap_proxy_1.TLVValues.PERMISSIONS, pairing.permission);
        });
        return Promise.resolve({
            data: tlv.encode(hap_proxy_1.TLVValues.STATE, hap_proxy_1.HAPStates.M2, ...tlvList),
        });
    }
    handleAccessories(connection, request) {
        if (!connection.pairingVerified) {
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        if (this.listenerCount(HAPServerEvents.ACCESSORIES) === 0) {
            return exports.NOT_FOUND_HANDLER(connection, request); // TODO adjust response
        }
        return new Promise((resolve, reject) => {
            this.emit(HAPServerEvents.ACCESSORIES, connection, once_1.once((error, response) => {
                if (error || !response) {
                    reject(error || new Error("Response was undefined!"));
                }
                else {
                    resolve(response);
                }
            }));
        });
    }
    handleCharacteristics(connection, request) {
        if (!connection.pairingVerified) {
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        if (request.method === http_protocol_1.HTTPMethod.GET) {
            if (this.listenerCount(HAPServerEvents.GET_CHARACTERISTICS) === 0) {
                return exports.NOT_FOUND_HANDLER(connection, request); // TODO adjust response
            }
            const query = url.parse(request.uri, true).query;
            return new Promise((resolve, reject) => {
                this.emit(HAPServerEvents.GET_CHARACTERISTICS, connection, query, once_1.once((error, response) => {
                    if (error || !response) {
                        reject(error || new Error("Response was undefined!"));
                    }
                    else {
                        resolve(response);
                    }
                }));
            });
        }
        else if (request.method === http_protocol_1.HTTPMethod.PUT) {
            if (this.listenerCount(HAPServerEvents.SET_CHARACTERISTICS) === 0) {
                return exports.NOT_FOUND_HANDLER(connection, request); // TODO adjust response
            }
            return new Promise((resolve, reject) => {
                this.emit(HAPServerEvents.SET_CHARACTERISTICS, connection, request.body, once_1.once((error, response) => {
                    if (error || !response) {
                        reject(error || new Error("Response was undefined!"));
                    }
                    else {
                        resolve(response);
                    }
                }));
            });
        }
        else {
            return Promise.reject();
            // TODO return appropriate error
        }
    }
    handlePrepare(connection, request) {
        if (!connection.pairingVerified) {
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        if (request.method == http_protocol_1.HTTPMethod.PUT) {
            if (this.listenerCount(HAPServerEvents.PREPARE_WRITE) === 0) {
                return exports.NOT_FOUND_HANDLER(connection, request); // TODO adjust response
            }
            return new Promise((resolve, reject) => {
                this.emit(HAPServerEvents.PREPARE_WRITE, connection, request.body, ((error, response) => {
                    if (error || !response) {
                        reject(error || new Error("Response was undefined!"));
                    }
                    else {
                        resolve(response);
                    }
                }));
            });
        }
        else {
            // TODO return appropriate error
            return Promise.reject();
        }
    }
    handleResource(connection, request) {
        if (!connection.pairingVerified) {
            return Promise.resolve({
                status: 470,
                contentType: http_protocol_1.HTTPContentType.HAP_JSON,
                data: Buffer.from(JSON.stringify({ status: hap_proxy_1.HAPStatusCode.INSUFFICIENT_PRIVILEGES })),
            });
        }
        if (request.method == http_protocol_1.HTTPMethod.POST) {
            if (this.listenerCount(HAPServerEvents.RESOURCE) === 0) {
                return exports.NOT_FOUND_HANDLER(connection, request); // TODO adjust response
            }
            return new Promise((resolve, reject) => {
                this.emit(HAPServerEvents.RESOURCE, connection, request.body, ((error, response) => {
                    if (error || !response) {
                        reject(error || new Error("Response was undefined!"));
                    }
                    else {
                        resolve(response);
                    }
                }));
            });
        }
        else {
            // TODO return appropriate error
            return Promise.reject();
        }
    }
}
exports.HAPServer = HAPServer;
var HAPServerConnectionEvents;
(function (HAPServerConnectionEvents) {
    HAPServerConnectionEvents["DISCONNECTED"] = "disconnected";
})(HAPServerConnectionEvents = exports.HAPServerConnectionEvents || (exports.HAPServerConnectionEvents = {}));
exports.NOT_FOUND_HANDLER = (connection, request) => {
    // TODO log connection name
    debugCon("Encountered 404 while accessing '%s'", request.uri);
    return Promise.resolve({
        status: http_protocol_1.HTTPStatus.NOT_FOUND,
        contentType: http_protocol_1.HTTPContentType.TEXT_HTML,
    });
};
class HAPServerConnection extends EventEmitter_1.EventEmitter {
    constructor(socket, routeHandler) {
        super();
        this.pairingVerified = false;
        this.processingRequest = false;
        this.disconnectAfterResponse = false;
        this.socketClosed = false;
        this.eventQueue = []; // queue of unencrypted events
        this.httpWorkingQueue = Promise.resolve();
        this.socket = socket;
        this.socket.on('data', this.handleData.bind(this));
        this.socket.on('close', this.handleClose.bind(this));
        this.socket.on('error', this.handleError.bind(this));
        this.parser = new http_protocol_1.HTTPRequestParser();
        this.routeHandler = routeHandler;
        this.remoteAddress = this.socket.remoteAddress + ":" + this.socket.remotePort;
        this.sessionID = uuid_1.uuid.generate(this.remoteAddress);
        this.socket.setNoDelay(true);
        this.socket.setKeepAlive(true);
    }
    disconnect() {
        if (this.socketClosed) {
            return;
        }
        this.pairingVerified = false;
        this.socketClosed = true;
        this.socket.end();
    }
    disconnectAfterWrite() {
        if (this.processingRequest) {
            this.disconnectAfterResponse = true;
        }
        else {
            this.disconnect();
        }
    }
    handleData(data) {
        if (this.socketClosed) {
            return;
        }
        if (this.processingRequest) { // TODO do we need this
            debugCon("Client tried sending http request while another request is still in progress");
            return;
        }
        this.processingRequest = true;
        if (this.encryptionContext) {
            try {
                data = encryption.layerDecrypt(data, this.encryptionContext);
            }
            catch (error) {
                this.disconnect();
                return;
            }
        }
        this.parser.appendData(data);
        const requests = this.parser.parse();
        requests.forEach(request => {
            const uri = request.uri;
            const route = uri.split("?")[0];
            const requestHandler = this.routeHandler[route];
            this.httpWorkingQueue = this.httpWorkingQueue
                .then(() => requestHandler ? requestHandler(this, request) : exports.NOT_FOUND_HANDLER(this, request))
                .catch(reason => {
                debugCon("Encountered error when handling response: " + reason);
                if (reason.stack) {
                    debugCon(reason.stack);
                }
                const response = {
                    status: 200,
                    contentType: this.pairingVerified
                        ? http_protocol_1.HTTPContentType.HAP_JSON
                        : http_protocol_1.HTTPContentType.PAIRING_TLV8,
                    data: this.pairingVerified
                        ? Buffer.from(JSON.stringify({
                            status: hap_proxy_1.HAPStatusCode.SERVICE_COMMUNICATION_FAILURE,
                        }))
                        : tlv.encode(hap_proxy_1.TLVValues.ERROR, hap_proxy_1.TLVErrors.UNKNOWN),
                };
                return Promise.resolve(response);
            })
                .then(response => this.sendResponse(response));
        });
    }
    sendResponse(response) {
        if (this.socketClosed) {
            throw new Error("Tried sending http response on a closed socket!");
        }
        const data = response.data || Buffer.alloc(0);
        const headers = response.headers || {};
        if (response.contentType && data.length > 0) {
            headers["Content-Type"] = response.contentType;
        }
        headers["Content-Length"] = data.length + "";
        let responseBuf = Buffer.concat([
            Buffer.from(`HTTP/1.1 ${response.status} OK\r\n` + // TODO status name
                Object.keys(headers).reduce((acc, header) => {
                    return acc + `${header}: ${headers[header]}\r\n`;
                }, "") +
                `\r\n` // additional newline before content
            ),
            data,
        ]);
        // with 'this.encryptionContext.decryptionNonce > 0' we ensure that we do not encrypt the
        // response of the last pair-verify response
        if (this.encryptionContext && this.encryptionContext.decryptionNonce > 0) {
            responseBuf = encryption.layerEncrypt(responseBuf, this.encryptionContext);
        }
        this.socket.write(responseBuf);
        this.processingRequest = false;
        this.flushEventQueue();
        if (this.disconnectAfterResponse) {
            this.disconnectAfterResponse = false;
            this.disconnect();
        }
    }
    sendRawEvent(data) {
        let eventBuf = Buffer.concat([
            Buffer.from(`EVENT/1.0 200 OK\r\n` +
                `Content-Type: ${http_protocol_1.HTTPContentType.HAP_JSON}\r\n` +
                `Content-Length: ${data.length}\r\n` +
                `\r\n` // additional newline before content
            ),
            data,
        ]);
        if (this.processingRequest) {
            this.eventQueue.push(eventBuf);
        }
        else {
            if (this.encryptionContext) {
                eventBuf = encryption.layerEncrypt(eventBuf, this.encryptionContext);
            }
            this.socket.write(eventBuf);
        }
    }
    flushEventQueue() {
        if (this.eventQueue.length > 0) {
            debug("Flushing event queue");
            this.eventQueue.forEach(eventBuf => {
                if (this.encryptionContext) {
                    eventBuf = encryption.layerEncrypt(eventBuf, this.encryptionContext);
                }
                this.socket.write(eventBuf);
            });
            this.eventQueue = [];
        }
    }
    handleClose() {
        // TODO log + other stuff?
        this.emit(HAPServerConnectionEvents.DISCONNECTED);
        // TODO somehow interrupt currently active chain
    }
    handleError(error) {
        console.log("Server client socket error:");
        console.log(error.stack);
        // TODO debug
    }
}
exports.HAPServerConnection = HAPServerConnection;
//# sourceMappingURL=HAPServer.js.map