"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataStreamClientConnection = exports.DataStreamClientConnectionEvents = exports.DataStreamClient = void 0;
const debug_1 = __importDefault(require("debug"));
const net_1 = __importDefault(require("net"));
const encryption = __importStar(require("../crypto/encryption"));
const HAPClient_1 = require("../HAPClient");
const DataStreamConnection_1 = require("./DataStreamConnection");
const debug = debug_1.default('DataStream:Client');
class DataStreamClient {
    constructor() {
        this.connections = [];
    }
    newConnection(clientConnection, host, port, salt) {
        const accessoryToControllerInfo = Buffer.from("HDS-Read-Encryption-Key");
        const controllerToAccessoryInfo = Buffer.from("HDS-Write-Encryption-Key");
        const accessoryToControllerEncryptionKey = encryption.HKDF("sha512", salt, clientConnection.encryptionContext.sharedSecret, accessoryToControllerInfo, 32);
        const controllerToAccessoryEncryptionKey = encryption.HKDF("sha512", salt, clientConnection.encryptionContext.sharedSecret, controllerToAccessoryInfo, 32);
        const socket = net_1.default.createConnection(port, host);
        const connection = new DataStreamClientConnection(socket, clientConnection, controllerToAccessoryEncryptionKey, accessoryToControllerEncryptionKey);
        connection.on(DataStreamConnection_1.DataStreamConnectionEvents.CLOSED, this.connectionClosed.bind(this, connection));
        this.connections.push(connection);
        return connection;
    }
    connectionClosed(connection) {
        debug("[%s] DataStream connection closed", connection.remoteAddress);
        const index = this.connections.indexOf(connection);
        if (index >= 0) {
            this.connections.splice(index, 1);
        }
        // TODO emit event(?)
    }
}
exports.DataStreamClient = DataStreamClient;
var DataStreamClientConnectionEvents;
(function (DataStreamClientConnectionEvents) {
    DataStreamClientConnectionEvents["CONNECTED"] = "connect";
})(DataStreamClientConnectionEvents = exports.DataStreamClientConnectionEvents || (exports.DataStreamClientConnectionEvents = {}));
class DataStreamClientConnection extends DataStreamConnection_1.DataStreamConnection {
    constructor(socket, connection, encryptionKey, decryptionKey) {
        super(socket);
        this.connection = connection;
        this.socket.on('connect', this.handleConnected.bind(this));
        this.connection.on(HAPClient_1.HAPClientConnectionEvents.DISCONNECTED, this.onHAPSessionClosed.bind(this)); // register close listener
        this.setKeys(encryptionKey, decryptionKey);
    }
    ensureConnected() {
        if (this.connected) {
            return Promise.resolve();
        }
        else if (this.connectPromise) {
            return this.connectPromise;
        }
        return this.connectPromise = new Promise((resolve, reject) => {
            this.socketConnectResolve = resolve;
            this.socketConnectReject = reject;
        });
    }
    handleConnected() {
        this.remoteAddress = this.socket.remoteAddress;
        this.connected = true;
        console.log("HDS Client is now connected"); // TODO adjust
        if (this.socketConnectResolve) {
            this.socketConnectResolve();
            this.socketConnectResolve = undefined;
            this.socketConnectReject = undefined;
        }
        this.emit(DataStreamClientConnectionEvents.CONNECTED);
        // TODO flush buffer queue
    }
    handleError(error) {
        super.handleError(error);
        if (this.socketConnectReject) {
            this.socketConnectReject();
            this.socketConnectResolve = undefined;
            this.socketConnectReject = undefined;
        }
    }
}
exports.DataStreamClientConnection = DataStreamClientConnection;
//# sourceMappingURL=DataStreamClient.js.map