"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataStreamProxy = void 0;
const debug_1 = __importDefault(require("debug"));
const DataStreamServer_1 = require("./DataStreamServer");
const DataStreamClient_1 = require("./DataStreamClient");
const crypto_1 = __importDefault(require("crypto"));
const DataStreamConnection_1 = require("./DataStreamConnection");
const debug = debug_1.default('DataStream:Proxy');
class DataStreamProxy {
    constructor() {
        this.dataStreamServer = new DataStreamServer_1.DataStreamServer();
        this.dataStreamClient = new DataStreamClient_1.DataStreamClient();
        this.dataStreamServer.on(DataStreamServer_1.DataStreamServerEvents.CONNECTION_IDENTIFIED, this.handleConnectionIdentified.bind(this));
    }
    generateKeySalt() {
        return crypto_1.default.randomBytes(32);
    }
    setupClient(hapConnection, host, listeningPort, salt) {
        const connection = this.dataStreamClient.newConnection(hapConnection, host, listeningPort, salt);
        return connection.ensureConnected().then(() => connection);
    }
    setupController(connection, controllerKeySalt, clientConnection) {
        return new Promise(resolve => {
            this.dataStreamServer.prepareSession(connection, controllerKeySalt, clientConnection, preparedSession => resolve(preparedSession));
        });
    }
    handleConnectionIdentified(preparedSession, serverConnection) {
        const clientConnection = preparedSession.hdsClientConnection;
        let lastTime = new Date().getTime();
        serverConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.PAYLOAD, DataStreamProxy.forwardPayload.bind(this, clientConnection));
        serverConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.MESSAGE, (message) => {
            const took = new Date().getTime() - lastTime;
            lastTime = new Date().getTime();
            console.log("Controller > Accessory: " + message.protocol + " " + message.topic + " +" + took + "ms");
            //fs.appendFileSync("hds-communication.txt", "Controller > Accessory: " + DataStreamProxy.stringify(message));
        });
        serverConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.CLOSED, DataStreamProxy.forwardClose.bind(this, clientConnection));
        clientConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.PAYLOAD, DataStreamProxy.forwardPayload.bind(this, serverConnection));
        clientConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.MESSAGE, (message) => {
            const took = new Date().getTime() - lastTime;
            lastTime = new Date().getTime();
            console.log("Accessory > Controller: " + message.protocol + " " + message.topic + " +" + took + "ms");
            //fs.appendFileSync("hds-communication.txt", "Accessory > Controller: " + DataStreamProxy.stringify(message));
        });
        clientConnection.on(DataStreamConnection_1.DataStreamConnectionEvents.CLOSED, DataStreamProxy.forwardClose.bind(this, serverConnection));
    }
    static stringify(value) {
        return JSON.stringify(value, (_, v) => {
            if (typeof v === 'bigint') {
                return v.toString();
            }
            else if (typeof v === "object" && v.type === "Buffer") {
                const data = v.data;
                return Buffer.from(data).toString("hex");
            }
            else {
                return v;
            }
        }, 4);
    }
    static forwardPayload(connection, payload) {
        //const printable = payload.toString("hex");
        //fs.appendFileSync("hds-communication.txt", printable);
        connection.sendRawHDSFrame(payload);
    }
    static forwardClose(connection) {
        connection.close();
    }
}
exports.DataStreamProxy = DataStreamProxy;
//# sourceMappingURL=DataStreamProxy.js.map