"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const simple_plist_1 = __importDefault(require("simple-plist"));
const uuid_1 = require("../utils/uuid");
const metadataFile = "/System/Library/PrivateFrameworks/HomeKitDaemon.framework/Resources/plain-metadata.config";
const plistData = simple_plist_1.default.readFileSync(metadataFile);
let characteristics;
let services;
try {
    characteristics = checkDefined(plistData.PlistDictionary.HAP.Characteristics);
    services = checkDefined(plistData.PlistDictionary.HAP.Services);
}
catch (error) {
    console.log("Unexpected structure of the plist file!");
    throw error;
}
const characteristicsOutput = fs_1.default.createWriteStream(path_1.default.join(__dirname, "CharacteristicType.ts"));
const serviceOutput = fs_1.default.createWriteStream(path_1.default.join(__dirname, "ServiceType.ts"));
writeBoth("// THIS FILE IS AUTO-GENERATED - DO NOT MODIFY\n");
writeBoth("\n");
/**
 * Characteristics
 */
characteristicsOutput.write("export enum CharacteristicType {\n");
characteristicsOutput.write("    // noinspection JSUnusedGlobalSymbols\n");
Object.entries(characteristics).forEach(([name, characteristic]) => {
    const id = uuid_1.uuid.toLongForm(characteristic.ShortUUID);
    const enumName = name.toUpperCase().replace(/-/g, "_").replace(/\./g, "_");
    characteristicsOutput.write(`    ${enumName} = "${id}",\n`);
});
characteristicsOutput.write("}\n");
writeTypeToName(characteristicsOutput, "CharacteristicType");
characteristicsOutput.end();
/**
 * Services
 */
serviceOutput.write("export enum ServiceType {\n");
serviceOutput.write("    // noinspection JSUnusedGlobalSymbols");
Object.entries(services).forEach(([name, service]) => {
    const id = uuid_1.uuid.toLongForm(service.ShortUUID);
    const enumName = name.toUpperCase().replace(/-/g, "_").replace(/\./g, "_");
    serviceOutput.write(`    ${enumName} = "${id}",\n`);
});
serviceOutput.write("}\n");
writeTypeToName(serviceOutput, "ServiceType");
serviceOutput.end();
function writeBoth(content) {
    characteristicsOutput.write(content);
    serviceOutput.write(content);
}
function checkDefined(input) {
    if (!input) {
        throw new Error("value is undefined!");
    }
    return input;
}
function writeTypeToName(output, name) {
    output.write(`
export namespace ${name}s {
    export const TYPE_TO_NAME: Record<string, string> = {};
}

for (let type in ${name}) {
    // @ts-ignore
    ${name}s.TYPE_TO_NAME[${name}[type]] = type;
}
`);
}
//# sourceMappingURL=generate.js.map