"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataStreamTransportManagementServiceFilter = exports.DataStreamTransportManagementServiceVersionCharacteristic = exports.DataStreamTransportManagementServiceSetupDataStreamTransportCharacteristicFilter = exports.AccessoryInformationServiceFilter = exports.AccessoryInformationServiceNameCharacteristicFilter = void 0;
const HAPServer_1 = require("./HAPServer");
const definitions_1 = require("./definitions");
const tlv = __importStar(require("./utils/tlv"));
const DataStreamProxy_1 = require("./datastream/DataStreamProxy");
const CharacteristicFilter_1 = require("./filters/CharacteristicFilter");
const ServiceFilter_1 = require("./filters/ServiceFilter");
const datastream_1 = require("./datastream");
/**
 * Service "Protocol Information"
 */
class AccessoryInformationServiceNameCharacteristicFilter extends CharacteristicFilter_1.CharacteristicFilterR {
    filterRead(connection, clientConnection, readValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.context.server.accessoryInfo.displayName; // override accessory name with the proxy server name
        });
    }
}
exports.AccessoryInformationServiceNameCharacteristicFilter = AccessoryInformationServiceNameCharacteristicFilter;
class AccessoryInformationServiceFilter extends ServiceFilter_1.ServiceFilter {
    constructor() {
        super(...arguments);
        this.characteristicFilterDefinitions = {
            [definitions_1.CharacteristicType.NAME]: AccessoryInformationServiceNameCharacteristicFilter,
        };
    }
}
exports.AccessoryInformationServiceFilter = AccessoryInformationServiceFilter;
// TODO hap version service (=> warning on incompatible version)
/**
 * Service "DataStreamManagement"
 */
class DataStreamTransportManagementServiceSetupDataStreamTransportCharacteristicFilter extends CharacteristicFilter_1.CharacteristicFilterControlPoint {
    constructor(context, parent, iid) {
        super(context, parent, iid);
        this.dataStreamProxy = new DataStreamProxy_1.DataStreamProxy();
        this.connectionContext = {};
    }
    filterRead(serverConnection, clientConnection, readValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const context = this.connectionContext[serverConnection.sessionID];
            return context ? context.lastResponse : "";
        });
    }
    filterWrite(serverConnection, clientConnection, writtenValue) {
        return __awaiter(this, void 0, void 0, function* () {
            let context = this.connectionContext[serverConnection.sessionID];
            if (!context) {
                context = {
                    lastResponse: "",
                    controllerKeySalt: Buffer.alloc(0),
                    proxyControllerKeySalt: Buffer.alloc(0),
                };
                this.connectionContext[serverConnection.sessionID] = context;
                // make sure we cleanup our stuff
                serverConnection.on(HAPServer_1.HAPServerConnectionEvents.DISCONNECTED, () => delete this.connectionContext[serverConnection.sessionID]);
            }
            const objects = tlv.decode(Buffer.from(writtenValue, "base64"));
            const sessionCommandType = objects[datastream_1.SetupDataStreamSessionTypes.SESSION_COMMAND_TYPE].readUInt8(0);
            const transportType = objects[datastream_1.SetupDataStreamSessionTypes.TRANSPORT_TYPE].readUInt8(0);
            // save the salt used for communication with the HomeKit controller device
            context.controllerKeySalt = objects[datastream_1.SetupDataStreamSessionTypes.CONTROLLER_KEY_SALT];
            if (sessionCommandType === datastream_1.SessionCommandType.START_SESSION) {
                if (transportType !== datastream_1.TransportType.HOMEKIT_DATA_STREAM) {
                    return writtenValue;
                }
                // generate the salt used for the communication with the accessory
                context.proxyControllerKeySalt = this.dataStreamProxy.generateKeySalt();
                // create our "fake" request to setup the HDS connection with the accessory
                const request = tlv.encode(datastream_1.SetupDataStreamSessionTypes.SESSION_COMMAND_TYPE, datastream_1.SessionCommandType.START_SESSION, datastream_1.SetupDataStreamSessionTypes.TRANSPORT_TYPE, datastream_1.TransportType.HOMEKIT_DATA_STREAM, datastream_1.SetupDataStreamSessionTypes.CONTROLLER_KEY_SALT, context.proxyControllerKeySalt);
                return request.toString("base64");
            }
            else {
                return writtenValue;
            }
        });
    }
    filterWriteResponse(serverConnection, clientConnection, writeResponseValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const context = this.connectionContext[serverConnection.sessionID];
            if (!context) {
                return "";
            }
            const objects = tlv.decode(Buffer.from(writeResponseValue, "base64"));
            const status = objects[datastream_1.SetupDataStreamWriteResponseTypes.STATUS].readUInt8(0);
            if (status !== datastream_1.DataStreamStatus.SUCCESS) {
                console.log("proxied accessory returned non successful status " + status); // TODO remove
                return writeResponseValue; // just forward non successful setup response
            }
            const sessionParametersData = objects[datastream_1.SetupDataStreamWriteResponseTypes.TRANSPORT_TYPE_SESSION_PARAMETERS];
            const accessoryKeySalt = objects[datastream_1.SetupDataStreamWriteResponseTypes.ACCESSORY_KEY_SALT];
            const sessionParametersTlv = tlv.decode(sessionParametersData);
            const port = sessionParametersTlv[datastream_1.TransportSessionConfiguration.TCP_LISTENING_PORT].readUInt16LE(0);
            const salt = Buffer.concat([context.proxyControllerKeySalt, accessoryKeySalt]);
            return this.dataStreamProxy.setupClient(clientConnection, this.context.client.deviceInfo.host, port, salt)
                .then(clientConnection => this.dataStreamProxy.setupController(serverConnection, context.controllerKeySalt, clientConnection))
                .then(preparedSession => {
                const listeningPort = tlv.encode(datastream_1.TransportSessionConfiguration.TCP_LISTENING_PORT, tlv.writeUInt16(preparedSession.port));
                let response = Buffer.concat([
                    tlv.encode(datastream_1.SetupDataStreamWriteResponseTypes.STATUS, datastream_1.DataStreamStatus.SUCCESS),
                    tlv.encode(datastream_1.SetupDataStreamWriteResponseTypes.TRANSPORT_TYPE_SESSION_PARAMETERS, listeningPort)
                ]);
                context.lastResponse = response.toString("base64"); // save last response without accessory key salt
                response = Buffer.concat([
                    response,
                    tlv.encode(datastream_1.SetupDataStreamWriteResponseTypes.ACCESSORY_KEY_SALT, preparedSession.accessoryKeySalt)
                ]);
                // console.log("Returning response to controller!"); // TODO message
                return response.toString("base64");
            }).catch(reason => {
                console.log("Could not create hds client: " + reason); // TODO message
                return tlv.encode(datastream_1.SetupDataStreamWriteResponseTypes.STATUS, datastream_1.DataStreamStatus.GENERIC_ERROR).toString('base64');
            });
        });
    }
}
exports.DataStreamTransportManagementServiceSetupDataStreamTransportCharacteristicFilter = DataStreamTransportManagementServiceSetupDataStreamTransportCharacteristicFilter;
class DataStreamTransportManagementServiceVersionCharacteristic extends CharacteristicFilter_1.CharacteristicFilterR {
    filterRead(serverConnection, clientConnection, readValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (readValue !== datastream_1.DataStream.VERSION) {
                console.log("WARNING! Data Stream endpoint of proxied accessory is not supported by hap-proxy!");
            }
            return readValue;
        });
    }
}
exports.DataStreamTransportManagementServiceVersionCharacteristic = DataStreamTransportManagementServiceVersionCharacteristic;
class DataStreamTransportManagementServiceFilter extends ServiceFilter_1.ServiceFilter {
    constructor(context, aid, iid) {
        super(context, aid, iid);
        this.characteristicFilterDefinitions = {
            [definitions_1.CharacteristicType.SETUP_DATA_STREAM_TRANSPORT]: DataStreamTransportManagementServiceSetupDataStreamTransportCharacteristicFilter,
            [definitions_1.CharacteristicType.VERSION]: DataStreamTransportManagementServiceVersionCharacteristic,
        };
    }
}
exports.DataStreamTransportManagementServiceFilter = DataStreamTransportManagementServiceFilter;
// TODO support RTP proxy (?)
//# sourceMappingURL=filters.js.map