import { ServiceFilter } from "./ServiceFilter";
import { HAPServerConnection } from "../HAPServer";
import { HAPProxy } from "../HAPProxy";
import { HAPClientConnection } from "../HAPClient";
/**
 * Read/write (with write response) characteristic filter
 */
export declare abstract class CharacteristicFilter<T> {
    readonly context: HAPProxy;
    readonly parent: ServiceFilter;
    readonly iid: number;
    /**
     * Creates a new instance of a CharacteristicFilter
     *
     * @param context {HAPProxy} - the associated proxy instance
     * @param parent {ServiceFilter} - the associated ServiceFilter instance
     * @param iid {number} - instance id of the characteristic
     */
    constructor(context: HAPProxy, parent: ServiceFilter, iid: number);
    abstract filterRead(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, readValue: T): Promise<T>;
    abstract filterWrite(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, writtenValue: T): Promise<T>;
    abstract filterWriteResponse(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, writeResponseValue: T): Promise<T>;
}
/**
 * Read/write characteristic filter
 */
export declare abstract class CharacteristicFilterRW<T> extends CharacteristicFilter<T> {
    filterWriteResponse(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, writeResponseValue: T): Promise<T>;
}
/**
 * Read only characteristic filter
 */
export declare abstract class CharacteristicFilterR<T> extends CharacteristicFilterRW<T> {
    filterWrite(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, writtenValue: T): Promise<T>;
}
/**
 * Write only characteristic filter
 */
export declare abstract class CharacteristicFilterW<T> extends CharacteristicFilterRW<T> {
    filterRead(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, readValue: T): Promise<T>;
}
/**
 * Control point characteristic filter
 */
export declare abstract class CharacteristicFilterControlPoint<T> extends CharacteristicFilter<T> {
    filterRead(serverConnection: HAPServerConnection, clientConnection: HAPClientConnection, readValue: T): Promise<T>;
}
/**
 * Creates a new instance of a CharacteristicFilter
 *
 * @param context {HAPProxy} - the associated proxy instance
 * @param parent {ServiceFilter} - the associated ServiceFilter instance
 * @param iid {number} - instance id of the characteristic
 */
export declare type CharacteristicFilterConstructor<T> = new (context: HAPProxy, parent: ServiceFilter, iid: number) => CharacteristicFilter<T>;
//# sourceMappingURL=CharacteristicFilter.d.ts.map