"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Advertiser = void 0;
const ciao_1 = __importDefault(require("@homebridge/ciao"));
const crypto_1 = __importDefault(require("crypto"));
const events_1 = require("events");
/**
 * Advertiser uses mdns to broadcast the presence of an Accessory to the local network.
 *
 * Note that as of iOS 9, an accessory can only pair with a single client. Instead of pairing your
 * accessories with multiple iOS devices in your home, Apple intends for you to use Home Sharing.
 * To support this requirement, we provide the ability to be "discoverable" or not (via a "service flag" on the
 * mdns payload).
 */
class Advertiser extends events_1.EventEmitter {
    constructor(accessoryInfo, responderOptions) {
        super();
        this.accessoryInfo = accessoryInfo;
        this.setupHash = this.computeSetupHash();
        this.responder = ciao_1.default.getResponder(responderOptions);
    }
    createService(port, serviceOptions) {
        this.advertisedService = this.responder.createService(Object.assign({ name: this.accessoryInfo.displayName, type: "hap" /* HAP */, txt: this.createTxt(), port: port }, serviceOptions));
        this.advertisedService.on("name-change" /* NAME_CHANGED */, this.emit.bind(this, "updated-name" /* UPDATED_NAME */));
    }
    startAdvertising() {
        return this.advertisedService.advertise();
    }
    updateAdvertisement() {
        this.advertisedService.updateTxt(this.createTxt());
    }
    destroyAdvertising() {
        return this.advertisedService.destroy();
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.destroyAdvertising(); // would also be done by the shutdown method below
            yield this.responder.shutdown();
            this.removeAllListeners();
        });
    }
    createTxt() {
        const statusFlags = [];
        if (!this.accessoryInfo.hasPairings()) {
            statusFlags.push(1 /* NOT_PAIRED */);
        }
        return {
            "c#": this.accessoryInfo.configNumber,
            ff: Advertiser.ff(),
            id: this.accessoryInfo.accessoryId,
            md: this.accessoryInfo.displayName,
            pv: Advertiser.protocolVersion,
            "s#": 1,
            sf: Advertiser.sf(...statusFlags),
            ci: this.accessoryInfo.category,
            sh: this.setupHash,
        };
    }
    computeSetupHash() {
        const hash = crypto_1.default.createHash('sha512');
        hash.update(this.accessoryInfo.setupID + this.accessoryInfo.accessoryId.toUpperCase());
        return hash.digest().slice(0, 4).toString('base64');
    }
    static ff(...flags) {
        let value = 0;
        flags.forEach(flag => value |= flag);
        return value;
    }
    static sf(...flags) {
        let value = 0;
        flags.forEach(flag => value |= flag);
        return value;
    }
}
exports.Advertiser = Advertiser;
Advertiser.protocolVersion = "1.1";
//# sourceMappingURL=Advertiser.js.map