"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HAPBonjourBrowser = exports.BonjourBrowserEvents = void 0;
const bonjour_hap_1 = __importDefault(require("bonjour-hap"));
const EventEmitter_1 = require("./EventEmitter");
const hap_proxy_1 = require("../types/hap-proxy");
var BonjourBrowserEvents;
(function (BonjourBrowserEvents) {
    BonjourBrowserEvents["UPDATE"] = "update";
    BonjourBrowserEvents["CONFIG_NUMBER_CHANGE"] = "config-change";
    BonjourBrowserEvents["PAIRED_CHANGE"] = "paired-change";
})(BonjourBrowserEvents = exports.BonjourBrowserEvents || (exports.BonjourBrowserEvents = {}));
class HAPBonjourBrowser extends EventEmitter_1.EventEmitter {
    constructor(clientInfo, mdnsConfig) {
        super();
        this.clientInfo = clientInfo;
        this.bonjourService = bonjour_hap_1.default(mdnsConfig);
        this.browser = this.bonjourService.find({
            type: "hap",
        }, this.handleUp.bind(this));
        this.promise = new Promise((resolve, reject) => {
            this.promiseResolver = resolve;
            this.timeout = setTimeout(() => {
                reject("HAP device '" + this.clientInfo.accessoryName + "' could not be found on the network!");
            }, HAPBonjourBrowser.TIMEOUT);
        });
    }
    handleUp(service) {
        if (service.name !== this.clientInfo.accessoryName) {
            return; // ignore if this is not the service we are searching for
        }
        if (!this.browser) {
            return;
        }
        this.browser.stop(); // bonjour-hap currently doesn't support events when txt records get updated
        this.browser = undefined;
        const txt = service.txt;
        const configNumber = txt["c#"];
        const statusFlag = txt["sf"];
        const protocolVersion = txt["pv"];
        const id = txt["id"];
        const category = txt["ci"];
        const paired = (statusFlag & hap_proxy_1.PairingStatusFlags.PAIRED) !== 0;
        let configNumberChanged = false;
        let pairedStatusChanged = false;
        if (this.deviceInfo) {
            configNumberChanged = configNumber !== this.deviceInfo.configNumber;
            pairedStatusChanged = paired !== this.deviceInfo.paired;
        }
        this.deviceInfo = {
            host: service.host,
            port: service.port,
            clientId: id,
            protocolVersion: protocolVersion,
            configNumber: configNumber,
            statusFlag: statusFlag,
            paired: paired,
            category: category,
        };
        if (this.promiseResolver) {
            this.promiseResolver(this.deviceInfo);
            this.promiseResolver = undefined;
            if (this.timeout) {
                clearTimeout(this.timeout);
                this.timeout = undefined;
            }
        }
        else {
            this.emit(BonjourBrowserEvents.UPDATE, this.deviceInfo);
            if (configNumberChanged) {
                this.emit(BonjourBrowserEvents.CONFIG_NUMBER_CHANGE, this.deviceInfo.configNumber);
            }
            if (pairedStatusChanged) {
                this.emit(BonjourBrowserEvents.PAIRED_CHANGE, this.deviceInfo.paired);
            }
        }
        if (!this.interval) {
            this.interval = setInterval(() => {
                this.browser = this.bonjourService.find({
                    type: "hap",
                }, this.handleUp.bind(this));
            }, HAPBonjourBrowser.INTERVAL);
        }
    }
    deviceInfoPromise() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.deviceInfo) {
                return this.deviceInfo;
            }
            else {
                return yield this.promise;
            }
        });
    }
}
exports.HAPBonjourBrowser = HAPBonjourBrowser;
HAPBonjourBrowser.TIMEOUT = 3000; // give it 3s to discover the service on the network TODO (?)
HAPBonjourBrowser.INTERVAL = 30000; // 30s
//# sourceMappingURL=HAPBonjourBrowser.js.map