/// <reference types="node" />
export declare enum ParsingState {
    HEAD = 0,
    HEADERS = 1,
    RESPONSE_BODY = 2
}
export declare enum HTTPMethod {
    GET = "GET",
    HEAD = "HEAD",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE",
    CONNECT = "CONNECT",
    OPTIONS = "options",
    TRACE = "TRACE"
}
export declare enum HTTPHeader {
    CONTENT_TYPE = "Content-Type",
    DATE = "Date",
    CONNECTION = "Connection",
    TRANSFER_ENCODING = "Transfer-Encoding",
    CONTENT_LENGTH = "Content-Length"
}
export declare enum HTTPStatus {
    SUCCESS = 200,
    NO_CONTENT = 204,
    MULTI_STATUS = 207,
    BAD_REQUEST = 400,
    NOT_FOUND = 404,
    UNPROCESSABLE_ENTITY = 422,
    INTERNAL_SERVER_ERROR = 500,
    SERVICE_UNAVAILABLE = 503
}
export declare enum HAPPairingHTTPStatus {
    OK = 200,
    BAD_REQUEST = 400,
    METHOD_NOT_ALLOWED = 405,
    TOO_MANY_REQUEST = 429,
    CONNECTION_AUTHORIZATION_REQUIRED = 470,
    INTERNAL_SERVER_ERROR = 500
}
export declare enum HTTPContentType {
    TEXT_HTML = "text/html",
    HAP_JSON = "application/hap+json",
    PAIRING_TLV8 = "application/pairing+tlv8"
}
export declare enum HTTPRoutes {
    IDENTIFY = "/identify",
    PAIR_SETUP = "/pair-setup",
    PAIR_VERIFY = "/pair-verify",
    PAIRINGS = "/pairings",
    ACCESSORIES = "/accessories",
    CHARACTERISTICS = "/characteristics",
    PREPARE = "/prepare",
    RESOURCE = "/resource"
}
export declare type HTTPResponse = {
    messageType: string;
    version: string;
    status: number;
    statusText: string;
    headers: Record<string, string>;
    body: Buffer;
};
export declare type HTTPRequest = {
    method: HTTPMethod;
    uri: string;
    version: string;
    headers: Record<string, string>;
    body: Buffer;
};
export declare type HTTPServerResponse = {
    status: HTTPStatus;
    contentType?: HTTPContentType;
    data?: Buffer;
    headers?: Record<string, string>;
};
export declare abstract class HTTPParser<T> {
    protected state: ParsingState;
    protected buffer: Buffer;
    protected readerIndex: number;
    protected constructor();
    appendData(buffer: Buffer): void;
    abstract parse(): T[];
    readLine(): Buffer | null;
    readFixedLengthLine(length: number): Buffer | null;
    readStringLine(): string | null;
    readChunk(): Buffer | null;
    readFixedLengthBuffer(length: number): Buffer | null;
}
export declare class HTTPResponseParser extends HTTPParser<HTTPResponse> {
    private static readonly HTTP_HEADER_PATTERN;
    private currentResponse;
    constructor();
    parse(): HTTPResponse[];
}
export declare class HTTPRequestParser extends HTTPParser<HTTPRequest> {
    private static readonly HTTP_HEADER_PATTERN;
    private currentRequest;
    constructor();
    parse(): HTTPRequest[];
}
//# sourceMappingURL=http-protocol.d.ts.map