"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientInfo = void 0;
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const storage_1 = require("./storage");
const uuid_1 = require("../utils/uuid");
const crypto = __importStar(require("crypto"));
class ClientInfo {
    constructor(accessoryName, pinProvider) {
        this.clientId = "";
        this.longTermPublicKey = Buffer.alloc(0);
        this.longTermSecretKey = Buffer.alloc(0);
        this.paired = false;
        this.accessoryIdentifier = "";
        this.accessoryLTPK = Buffer.alloc(0); // currently we only support one pairing
        this.loaded = false;
        this.accessoryName = accessoryName;
        this.pinProvider = pinProvider;
    }
    pincode() {
        if (typeof this.pinProvider === "string") {
            return Promise.resolve(this.pinProvider);
        }
        else {
            const callable = this.pinProvider;
            return new Promise(resolve => callable(pinCode => resolve(pinCode)));
        }
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.loaded) {
                throw new Error("Tried saving ClientInfo before it was even loaded!");
            }
            const saved = {
                clientId: this.clientId,
                longTermPublicKey: this.longTermPublicKey.toString("hex"),
                longTermSecretKey: this.longTermSecretKey.toString("hex"),
                paired: this.paired,
                accessoryIdentifier: this.paired ? this.accessoryIdentifier : undefined,
                accessoryLTPK: this.paired ? this.accessoryLTPK.toString("hex") : undefined,
            };
            const storageKey = storage_1.StorageManager.clientFormatPersistKey(this.accessoryName);
            yield storage_1.StorageManager.init();
            yield storage_1.StorageManager.setItem(storageKey, saved);
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.loaded) {
                return Promise.resolve();
            }
            const storageKey = storage_1.StorageManager.clientFormatPersistKey(this.accessoryName);
            yield storage_1.StorageManager.init();
            const saved = yield storage_1.StorageManager.getItem(storageKey);
            this.loaded = true;
            if (saved) {
                this.clientId = saved.clientId;
                this.longTermPublicKey = Buffer.from(saved.longTermPublicKey, "hex");
                this.longTermSecretKey = Buffer.from(saved.longTermSecretKey, "hex");
                if (saved.paired) {
                    this.paired = true;
                    this.accessoryIdentifier = saved.accessoryIdentifier;
                    this.accessoryLTPK = Buffer.from(saved.accessoryLTPK, "hex");
                }
            }
            else {
                this.clientId = uuid_1.uuid.generate(crypto.randomBytes(4)).toUpperCase(); // generate random clientId
                const longTerm = tweetnacl_1.default.sign.keyPair();
                this.longTermPublicKey = Buffer.from(longTerm.publicKey);
                this.longTermSecretKey = Buffer.from(longTerm.secretKey);
                yield this.save();
            }
        });
    }
}
exports.ClientInfo = ClientInfo;
//# sourceMappingURL=ClientInfo.js.map