"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IdentifierCache = void 0;
const storage_1 = require("./storage");
// mapping of 'accessoryId.instanceId' to characteristics type (aka uuid)
class IdentifierCache {
    constructor(clientInfo) {
        this.typeCache = {}; // indexed by "aid.iid" (iid of the characteristic)
        this.serviceIidCache = {}; // indexed by "aid.iid" (iid of the characteristic); value is iid of the service
        this.loaded = false;
        this.clientInfo = clientInfo;
    }
    rebuild() {
        this.typeCache = {};
    }
    persists(aid, serviceIid, characteristicIid, serviceType, characteristicType) {
        this.typeCache[aid + "." + characteristicIid] = {
            serviceType: serviceType,
            characteristicType: characteristicType,
        };
        this.serviceIidCache[aid + "." + characteristicIid] = serviceIid;
    }
    entries() {
        return Object.entries(this.typeCache).map(([id, types]) => {
            const [aid, iid] = id.split("|");
            const serviceIid = this.serviceIidCache[id];
            return {
                aid: parseInt(aid),
                serviceIid: serviceIid,
                characteristicIid: parseInt(iid),
                serviceType: types.serviceType,
                characteristicType: types.characteristicType,
            };
        });
    }
    /**
     * Lookup of the service and characteristic types for the given ids
     *
     * @param aid {number} - accessory id
     * @param iid {number} - instance id of the characteristic
     */
    lookupType(aid, iid) {
        return this.typeCache[aid + "." + iid];
    }
    /**
     * Lookup of the iid of the service for the given ids
     *
     * @param aid {number} - accessory id
     * @param iid {number} - instance id of the characteristic
     */
    lookupServiceIid(aid, iid) {
        return this.serviceIidCache[aid + "." + iid];
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.loaded) {
                throw new Error("Tried saving ClientInfo before it was even loaded!");
            }
            yield this.clientInfo.load();
            const storageKey = storage_1.StorageManager.identifierCacheFormatPersistKey(this.clientInfo.clientId);
            yield storage_1.StorageManager.init();
            const saved = {
                typeCache: this.typeCache,
                serviceIidCache: this.serviceIidCache,
            };
            yield storage_1.StorageManager.setItem(storageKey, saved);
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.clientInfo.load();
            const storageKey = storage_1.StorageManager.identifierCacheFormatPersistKey(this.clientInfo.clientId);
            yield storage_1.StorageManager.init();
            const saved = yield storage_1.StorageManager.getItem(storageKey);
            this.loaded = true;
            if (saved) {
                this.typeCache = saved.typeCache || {};
                this.serviceIidCache = saved.serviceIidCache || {};
            }
            else {
                yield this.save();
            }
        });
    }
}
exports.IdentifierCache = IdentifierCache;
//# sourceMappingURL=IdentifierCache.js.map