"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HAPEncryptionContext = exports.PairingStatusFlags = exports.HAPAccessoryCategory = exports.HAPStatusCode = exports.TLVErrors = exports.HAPStates = exports.PairMethods = exports.PairingFlags = exports.TLVValues = void 0;
var TLVValues;
(function (TLVValues) {
    // noinspection JSUnusedGlobalSymbols
    TLVValues[TLVValues["METHOD"] = 0] = "METHOD";
    TLVValues[TLVValues["IDENTIFIER"] = 1] = "IDENTIFIER";
    TLVValues[TLVValues["SALT"] = 2] = "SALT";
    TLVValues[TLVValues["PUBLIC_KEY"] = 3] = "PUBLIC_KEY";
    TLVValues[TLVValues["PASSWORD_PROOF"] = 4] = "PASSWORD_PROOF";
    TLVValues[TLVValues["ENCRYPTED_DATA"] = 5] = "ENCRYPTED_DATA";
    TLVValues[TLVValues["STATE"] = 6] = "STATE";
    TLVValues[TLVValues["ERROR"] = 7] = "ERROR";
    TLVValues[TLVValues["RETRY_DELAY"] = 8] = "RETRY_DELAY";
    TLVValues[TLVValues["CERTIFICATE"] = 9] = "CERTIFICATE";
    TLVValues[TLVValues["SIGNATURE"] = 10] = "SIGNATURE";
    TLVValues[TLVValues["PERMISSIONS"] = 11] = "PERMISSIONS";
    TLVValues[TLVValues["FRAGMENT_DATA"] = 12] = "FRAGMENT_DATA";
    TLVValues[TLVValues["FRAGMENT_LAST"] = 13] = "FRAGMENT_LAST";
    TLVValues[TLVValues["PAIRING_FLAGS"] = 19] = "PAIRING_FLAGS";
    TLVValues[TLVValues["SEPARATOR"] = 255] = "SEPARATOR"; // Zero-length TLV that separates different TLVs in a list.
})(TLVValues = exports.TLVValues || (exports.TLVValues = {}));
// noinspection JSUnusedGlobalSymbols
var PairingFlags;
(function (PairingFlags) {
    // noinspection JSUnusedGlobalSymbols
    PairingFlags[PairingFlags["TRANSIENT_PAIR_SETUP"] = 16] = "TRANSIENT_PAIR_SETUP";
    PairingFlags[PairingFlags["SPLIT_PAIR_SETUP"] = 16777216] = "SPLIT_PAIR_SETUP";
})(PairingFlags = exports.PairingFlags || (exports.PairingFlags = {}));
var PairMethods;
(function (PairMethods) {
    // noinspection JSUnusedGlobalSymbols
    PairMethods[PairMethods["PAIR_SETUP"] = 0] = "PAIR_SETUP";
    PairMethods[PairMethods["PAIR_SETUP_WITH_AUTH"] = 1] = "PAIR_SETUP_WITH_AUTH";
    PairMethods[PairMethods["PAIR_VERIFY"] = 2] = "PAIR_VERIFY";
    PairMethods[PairMethods["ADD_PAIRING"] = 3] = "ADD_PAIRING";
    PairMethods[PairMethods["REMOVE_PAIRING"] = 4] = "REMOVE_PAIRING";
    PairMethods[PairMethods["LIST_PAIRINGS"] = 5] = "LIST_PAIRINGS";
})(PairMethods = exports.PairMethods || (exports.PairMethods = {}));
var HAPStates;
(function (HAPStates) {
    HAPStates[HAPStates["M1"] = 1] = "M1";
    HAPStates[HAPStates["M2"] = 2] = "M2";
    HAPStates[HAPStates["M3"] = 3] = "M3";
    HAPStates[HAPStates["M4"] = 4] = "M4";
    HAPStates[HAPStates["M5"] = 5] = "M5";
    HAPStates[HAPStates["M6"] = 6] = "M6";
})(HAPStates = exports.HAPStates || (exports.HAPStates = {}));
// Error codes and the like, guessed by packet inspection
var TLVErrors;
(function (TLVErrors) {
    // noinspection JSUnusedGlobalSymbols
    TLVErrors[TLVErrors["UNKNOWN"] = 1] = "UNKNOWN";
    TLVErrors[TLVErrors["INVALID_REQUEST"] = 2] = "INVALID_REQUEST";
    TLVErrors[TLVErrors["AUTHENTICATION"] = 2] = "AUTHENTICATION";
    TLVErrors[TLVErrors["BACKOFF"] = 3] = "BACKOFF";
    TLVErrors[TLVErrors["MAX_PEERS"] = 4] = "MAX_PEERS";
    TLVErrors[TLVErrors["MAX_TRIES"] = 5] = "MAX_TRIES";
    TLVErrors[TLVErrors["UNAVAILABLE"] = 6] = "UNAVAILABLE";
    TLVErrors[TLVErrors["BUSY"] = 7] = "BUSY"; // cannot accept pairing request at this time
})(TLVErrors = exports.TLVErrors || (exports.TLVErrors = {}));
// noinspection JSUnusedGlobalSymbols
var HAPStatusCode;
(function (HAPStatusCode) {
    // noinspection JSUnusedGlobalSymbols
    HAPStatusCode[HAPStatusCode["SUCCESS"] = 0] = "SUCCESS";
    HAPStatusCode[HAPStatusCode["INSUFFICIENT_PRIVILEGES"] = -70401] = "INSUFFICIENT_PRIVILEGES";
    HAPStatusCode[HAPStatusCode["SERVICE_COMMUNICATION_FAILURE"] = -70402] = "SERVICE_COMMUNICATION_FAILURE";
    HAPStatusCode[HAPStatusCode["RESOURCE_BUSY"] = -70403] = "RESOURCE_BUSY";
    HAPStatusCode[HAPStatusCode["READ_ONLY_CHARACTERISTIC"] = -70404] = "READ_ONLY_CHARACTERISTIC";
    HAPStatusCode[HAPStatusCode["WRITE_ONLY_CHARACTERISTIC"] = -70405] = "WRITE_ONLY_CHARACTERISTIC";
    HAPStatusCode[HAPStatusCode["NOTIFICATION_NOT_SUPPORTED"] = -70406] = "NOTIFICATION_NOT_SUPPORTED";
    HAPStatusCode[HAPStatusCode["OUT_OF_RESOURCE"] = -70407] = "OUT_OF_RESOURCE";
    HAPStatusCode[HAPStatusCode["OPERATION_TIMED_OUT"] = -70408] = "OPERATION_TIMED_OUT";
    HAPStatusCode[HAPStatusCode["RESOURCE_DOES_NOT_EXIST"] = -70409] = "RESOURCE_DOES_NOT_EXIST";
    HAPStatusCode[HAPStatusCode["INVALID_VALUE_IN_REQUEST"] = -70410] = "INVALID_VALUE_IN_REQUEST";
    HAPStatusCode[HAPStatusCode["INSUFFICIENT_AUTHORIZATION"] = -70411] = "INSUFFICIENT_AUTHORIZATION";
})(HAPStatusCode = exports.HAPStatusCode || (exports.HAPStatusCode = {}));
var HAPAccessoryCategory;
(function (HAPAccessoryCategory) {
    // noinspection JSUnusedGlobalSymbols
    HAPAccessoryCategory[HAPAccessoryCategory["OTHER"] = 1] = "OTHER";
    HAPAccessoryCategory[HAPAccessoryCategory["BRIDGE"] = 2] = "BRIDGE";
    HAPAccessoryCategory[HAPAccessoryCategory["FAN"] = 3] = "FAN";
    HAPAccessoryCategory[HAPAccessoryCategory["GARAGE_DOOR_OPENER"] = 4] = "GARAGE_DOOR_OPENER";
    HAPAccessoryCategory[HAPAccessoryCategory["LIGHTBULB"] = 5] = "LIGHTBULB";
    HAPAccessoryCategory[HAPAccessoryCategory["DOOR_LOCK"] = 6] = "DOOR_LOCK";
    HAPAccessoryCategory[HAPAccessoryCategory["OUTLET"] = 7] = "OUTLET";
    HAPAccessoryCategory[HAPAccessoryCategory["SWITCH"] = 8] = "SWITCH";
    HAPAccessoryCategory[HAPAccessoryCategory["THERMOSTAT"] = 9] = "THERMOSTAT";
    HAPAccessoryCategory[HAPAccessoryCategory["SENSOR"] = 10] = "SENSOR";
    HAPAccessoryCategory[HAPAccessoryCategory["ALARM_SYSTEM"] = 11] = "ALARM_SYSTEM";
    HAPAccessoryCategory[HAPAccessoryCategory["SECURITY_SYSTEM"] = 11] = "SECURITY_SYSTEM";
    HAPAccessoryCategory[HAPAccessoryCategory["DOOR"] = 12] = "DOOR";
    HAPAccessoryCategory[HAPAccessoryCategory["WINDOW"] = 13] = "WINDOW";
    HAPAccessoryCategory[HAPAccessoryCategory["WINDOW_COVERING"] = 14] = "WINDOW_COVERING";
    HAPAccessoryCategory[HAPAccessoryCategory["PROGRAMMABLE_SWITCH"] = 15] = "PROGRAMMABLE_SWITCH";
    HAPAccessoryCategory[HAPAccessoryCategory["RANGE_EXTENDER"] = 16] = "RANGE_EXTENDER";
    HAPAccessoryCategory[HAPAccessoryCategory["CAMERA"] = 17] = "CAMERA";
    HAPAccessoryCategory[HAPAccessoryCategory["IP_CAMERA"] = 17] = "IP_CAMERA";
    HAPAccessoryCategory[HAPAccessoryCategory["VIDEO_DOORBELL"] = 18] = "VIDEO_DOORBELL";
    HAPAccessoryCategory[HAPAccessoryCategory["AIR_PURIFIER"] = 19] = "AIR_PURIFIER";
    HAPAccessoryCategory[HAPAccessoryCategory["AIR_HEATER"] = 20] = "AIR_HEATER";
    HAPAccessoryCategory[HAPAccessoryCategory["AIR_CONDITIONER"] = 21] = "AIR_CONDITIONER";
    HAPAccessoryCategory[HAPAccessoryCategory["AIR_HUMIDIFIER"] = 22] = "AIR_HUMIDIFIER";
    HAPAccessoryCategory[HAPAccessoryCategory["AIR_DEHUMIDIFIER"] = 23] = "AIR_DEHUMIDIFIER";
    HAPAccessoryCategory[HAPAccessoryCategory["APPLE_TV"] = 24] = "APPLE_TV";
    HAPAccessoryCategory[HAPAccessoryCategory["HOMEPOD"] = 25] = "HOMEPOD";
    HAPAccessoryCategory[HAPAccessoryCategory["SPEAKER"] = 26] = "SPEAKER";
    HAPAccessoryCategory[HAPAccessoryCategory["AIRPORT"] = 27] = "AIRPORT";
    HAPAccessoryCategory[HAPAccessoryCategory["SPRINKLER"] = 28] = "SPRINKLER";
    HAPAccessoryCategory[HAPAccessoryCategory["FAUCET"] = 29] = "FAUCET";
    HAPAccessoryCategory[HAPAccessoryCategory["SHOWER_HEAD"] = 30] = "SHOWER_HEAD";
    HAPAccessoryCategory[HAPAccessoryCategory["TELEVISION"] = 31] = "TELEVISION";
    HAPAccessoryCategory[HAPAccessoryCategory["TARGET_CONTROLLER"] = 32] = "TARGET_CONTROLLER";
    HAPAccessoryCategory[HAPAccessoryCategory["ROUTER"] = 33] = "ROUTER"; // HomeKit enabled router
})(HAPAccessoryCategory = exports.HAPAccessoryCategory || (exports.HAPAccessoryCategory = {}));
var PairingStatusFlags;
(function (PairingStatusFlags) {
    // noinspection JSUnusedGlobalSymbols
    PairingStatusFlags[PairingStatusFlags["PAIRED"] = 1] = "PAIRED";
    PairingStatusFlags[PairingStatusFlags["NOT_JOINED_WIFI"] = 2] = "NOT_JOINED_WIFI";
    PairingStatusFlags[PairingStatusFlags["PROBLEM"] = 4] = "PROBLEM";
})(PairingStatusFlags = exports.PairingStatusFlags || (exports.PairingStatusFlags = {}));
class HAPEncryptionContext {
    constructor(sharedSecret, encryptionKey, decryptionKey) {
        this.encryptionNonce = 0;
        this.decryptionNonce = 0;
        this.sharedSecret = sharedSecret;
        this.encryptionKey = encryptionKey;
        this.decryptionKey = decryptionKey;
    }
}
exports.HAPEncryptionContext = HAPEncryptionContext;
//# sourceMappingURL=hap-proxy.js.map