"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.uuid = void 0;
const crypto_1 = __importDefault(require("crypto"));
var uuid;
(function (uuid_1) {
    uuid_1.BASE_UUID = '-0000-1000-8000-0026BB765291';
    const VALID_UUID_REGEX = /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/i;
    const SHORT_FORM_REGEX = /^0*([0-9a-f]{1,8})-([0-9a-f]{4}-){3}[0-9a-f]{12}$/i;
    const VALID_SHORT_REGEX = /^[0-9a-f]{1,8}$/i;
    // http://stackoverflow.com/a/25951500/66673
    function generate(data) {
        const sha1sum = crypto_1.default.createHash('sha1');
        sha1sum.update(data);
        const s = sha1sum.digest('hex');
        let i = -1;
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
            i += 1;
            switch (c) {
                case 'y':
                    return ((parseInt('0x' + s[i], 16) & 0x3) | 0x8).toString(16);
                case 'x':
                default:
                    return s[i];
            }
        });
    }
    uuid_1.generate = generate;
    function isValid(UUID) {
        return VALID_UUID_REGEX.test(UUID);
    }
    uuid_1.isValid = isValid;
    // https://github.com/defunctzombie/node-uuid/blob/master/uuid.js
    function unparse(buf, offset = 0) {
        let i = offset;
        return buf[i++].toString(16) + buf[i++].toString(16) +
            buf[i++].toString(16) + buf[i++].toString(16) + '-' +
            buf[i++].toString(16) + buf[i++].toString(16) + '-' +
            buf[i++].toString(16) + buf[i++].toString(16) + '-' +
            buf[i++].toString(16) + buf[i++].toString(16) + '-' +
            buf[i++].toString(16) + buf[i++].toString(16) +
            buf[i++].toString(16) + buf[i++].toString(16) +
            buf[i++].toString(16) + buf[i++].toString(16);
    }
    uuid_1.unparse = unparse;
    function write(uuid, buf, offset = 0) {
        uuid = uuid.replace(/-/g, "");
        for (let i = 0; i < uuid.length; i += 2) {
            const octet = uuid.substring(i, i + 2);
            buf.write(octet, offset++, "hex");
        }
    }
    uuid_1.write = write;
    function toShortForm(uuid, base = uuid_1.BASE_UUID) {
        if (!isValid(uuid))
            throw new TypeError('uuid was not a valid UUID or short form UUID');
        if (base && !isValid('00000000' + base))
            throw new TypeError('base was not a valid base UUID');
        if (base && !uuid.endsWith(base))
            return uuid.toUpperCase();
        return uuid.replace(SHORT_FORM_REGEX, '$1').toUpperCase();
    }
    uuid_1.toShortForm = toShortForm;
    function toLongForm(uuid, base = uuid_1.BASE_UUID) {
        if (isValid(uuid))
            return uuid.toUpperCase();
        if (!VALID_SHORT_REGEX.test(uuid))
            throw new TypeError('uuid was not a valid UUID or short form UUID (' + uuid + ')');
        if (!isValid('00000000' + base))
            throw new TypeError('base was not a valid base UUID');
        return (('00000000' + uuid).substr(-8) + base).toUpperCase();
    }
    uuid_1.toLongForm = toLongForm;
})(uuid = exports.uuid || (exports.uuid = {}));
//# sourceMappingURL=uuid.js.map